###########################################
### function to compute inv and lndet...
###########################################
lndetANDinvCb <- function(C , b = NULL){
    n <- nrow(C)
### some issue with dgeMatrix matrices, chol(C) was different to cholC[1:n1:n], so...    
    cholC <- try(chol(C[1:n,1:n,drop = FALSE]) , silent = TRUE)
    if (is.character(cholC)){
        lndetC <- invCb <- NA
    }else{
        lndetC <- 2 * sum(log(diag(cholC)))
        if (is.null(b)){        
            invCb <- chol2inv(cholC)
        }else{
### test if C is sparse...
          if(is.matrix(C)){    
### most efficient way if dense...
            invCb<- backsolve(cholC , forwardsolve(t(cholC) , b))   
          }else{
### but above doesn't work for sparse matrices, so...          
            invCb <- chol2inv(cholC) %*% b 
          }
        }
   }

   return(list('lndetC' = lndetC , 'invCb' = invCb, 'cholC' = cholC))
}

###########################################
### integrated lik fn a (over beta)...
### assuming uniform (improper) prior for beta... 
### equiv to reml function...
###########################################
intLika <- function(pars , z , X , intLikbObj , covModel = 'ExponentialC' , nu = 0.5){
  c1<-as.numeric(pars[1])
  a<-as.numeric(pars[2])

  if ((c1 > 0) & (a > 0)){
    N <- length(z)
    p <- dim(X)[[2]]

    if (missing(intLikbObj)){
### make DMats.a2a in main script, and get it from there.
      Cc <- averageCov(covModel = covModel , covPars = c(c1 , a) , DMats = DMats.a2a , nu = nu)

      tmp <- lndetANDinvCb(Cc , cbind(z,X))

      if(is.na(tmp$lndetC)){ return(-Inf) }else{}

      lndetC <- tmp$lndetC
      invCz <- tmp$invCb[,1,drop = FALSE]
      invCX <- tmp$invCb[,-1,drop = FALSE]
      
      XinvCX <- t(X) %*% invCX
      XinvCz <- t(X) %*% invCz
      zinvCz <- t(z) %*% invCz

      if(p > 0){
        tmp <- lndetANDinvCb(XinvCX , XinvCz)
        lndetXinvCX <- tmp$lndetC
        betahat <- tmp$invCb
        resinvCres <- zinvCz - t(XinvCz) %*% betahat
      }else{
        lndetXinvCX <- c()
        betahat <- c()
        resinvCres <- zinvCz
      }
    }else{
      lndetC <- N * log(c1) + intLikbObj$lndetA
      lndetXinvCX <- -p * log(c1) + intLikbObj$lndetXinvAX
      resinvCres <- intLikbObj$resinvAres / c1
    }

    logLik <- -0.5 * ((N - p) * log(2 * pi) + lndetC + lndetXinvCX + resinvCres)

  }else{
    logLik <- -Inf
  }
  
  return(logLik)
}

###########################################
### integrated lik fn b (over beta and sigma2)...
### assuming uniform (improper) prior for beta... 
### and uniform (improper) prior for ln c1 (ie f0(c1) = 1 / c1)
###########################################
intLikb <- function(pars , z , X , covModel = 'ExponentialC' , nu = 0.5){

  a<-as.numeric(pars[1])

  listOut <- list('logLik' = -Inf , 'Ac' = NA , 'lndetA' = NA , 'invA' = NA , 
        'invAX' = NA , 'invAz' = NA , 'lndetXinvAX' = NA , 'invXinvAX' = NA ,
        'resinvAres' = NA , 'betahat' = NA , 'sigma2hat' = NA)

  if (a > 0){
### make DMats.a2a in main script, and get it from there.
    Ac <- averageCov(covModel = covModel , covPars = c(1 , a) , DMats = DMats.a2a , nu = nu)

    N <- length(z)
    p <- dim(X)[[2]]

    tmp <- lndetANDinvCb(Ac , cbind(z,X))

    if(is.na(tmp$lndetC)){ return(listOut) }else{}

    lndetA <- tmp$lndetC
    invAz <- tmp$invCb[,1,drop = FALSE]
    invAX <- tmp$invCb[,-1,drop = FALSE]

    invA <- chol2inv(tmp$cholC)

    XinvAX <- t(X) %*% invAX
    XinvAz <- t(X) %*% invAz
    zinvAz <- t(z) %*% invAz
    
    if (p > 0){
      tmp <- lndetANDinvCb(XinvAX , XinvAz)
    
      if(is.na(tmp$lndetC)){ return(listOut) }else{}

      lndetXinvAX <- tmp$lndetC
      betahat <- tmp$invCb
      invXinvAX <- chol2inv(tmp$cholC)

      resinvAres <- as.numeric(zinvAz - t(XinvAz) %*% betahat)
    }else{
      lndetXinvAX <- 0
      betahat <- c()
      invXinvAX <- c()
      resinvAres <- as.numeric(zinvAz)
    }
    sigma2hat <- (resinvAres + 2 * priorc1Pars[2]) / (N - p + 2 * priorc1Pars[1])
    
    const <- -0.5 * (N - p) * log(pi) + lgamma((N - p + 2 * priorc1Pars[1]) / 2) + priorc1Pars[1] * log(2)
		
    #browser()
    
    logLik <- const - 0.5 * (lndetA + lndetXinvAX + (N - p + 2 * priorc1Pars[1]) * log(resinvAres + 2 * priorc1Pars[2]))

    listOut <- list('logLik' = logLik , 'Ac' = Ac , 'lndetA' = lndetA , 'invA' = invA , 'invAX' = invAX , 'invAz' = invAz , 
		  'lndetXinvAX' = lndetXinvAX , 'invXinvAX' = invXinvAX , 'resinvAres' = resinvAres , 'betahat' = betahat , 'sigma2hat' = sigma2hat)
  }else{}

  return(listOut)
}

##################################
### a term that is proportional to the marginal post for each betai (integrated over beta_i and sigma2)
### given any particular value of a
### with some terms that depend on a...
##################################
margbetab <- function(pars , z , X , intLikbObj , betaMat){
### row ip of betaMat is vector of values for betai (assume all the same length so can be stored in a matrix)
  a<-as.numeric(pars[1])
  N <- length(z)
  p <- dim(X)[[2]]
  
  if ((a > 0) & (p > 0)){

    if (!is.matrix(betaMat)){
      betaMat <- matrix(betaMat , p , length(betaMat) / p)  
    }else{}
    
    if(all(!is.na(intLikbObj$invXinvAX))){
      vbetahat <- intLikbObj$sigma2hat * diag(intLikbObj$invXinvAX)
      betahat <- intLikbObj$betahat
    }else{
      vbetahat <- matrix(NA , p , 1)
      betahat <- matrix(NA , p , 1)
    }

### matrix for making replications...    
    mTmp <- matrix(1 , 1 , dim(betaMat)[[2]])

### multivariate t dist with N-p df (see version in Roth, on the mv t dist), marginals also t with N-p df...
### lnconst should be p x 1, will require replication for combining with betaMat...    
    lnconst <- lgamma(0.5*(N-p+2*priorc1Pars[1]+1)) - lgamma(0.5*(N-p+2*priorc1Pars[1])) - 
            0.5 * log((N-p+2*priorc1Pars[1]) * pi) - 0.5 * log(vbetahat)

    lnmpb <- kronecker(lnconst , mTmp) -
      0.5 * (N - p + 2 * priorc1Pars[1] + 1) * log(1 + (1/(N-p+ 2 * priorc1Pars[1])) * ((betaMat - kronecker(betahat , mTmp)) ^ 2) / kronecker(vbetahat , mTmp))
    
  }else{
    lnmpb <- NA
  }
  
  return(list('lnmpb' = lnmpb))
}

##################################
### define prior for a...
##################################
priora <- function(a , X){
  if (prioraType == 'invgamma'){
    priora<-log(dinvgamma(a,prioraPars[1],prioraPars[2])) 
  }else if (prioraType == 'uni'){
    amin <- prioraPars[1]
    amax <- prioraPars[2]

    iTmp <- which((a > amin) & (a < amax))
    priora <- NA * numeric(length(a))
    priora[iTmp] <- -log(amax - amin)
    priora[-iTmp] <- -Inf
  }else{
    stop('Set prioraType = "invgamma", "uni" or "reference" ')
  }
  
  return(priora)
}

##################################
### function to set default param vecs for c1, a and beta...
##################################
setDefaultParamVecs <- function(X , z){
  aVec <- seq(1,100,1)
  c1Vec <- seq(1 , 100 , 1)

  N <- dim(X)[[1]]
  p <- dim(X)[[2]]
  betaTmp <- solve(t(X) %*% X , t(X) %*% z)
  sigma2Tmp <- sum((z - X %*% betaTmp) ^ 2) / (N - p)
  vbetaTmp <- as.numeric(sigma2Tmp) * diag(solve(t(X) %*% X))
  stdNrm <- seq(-6 , 6 , 0.05)
  betaMat <- kronecker(betaTmp , matrix(1 , 1 , length(stdNrm))) + kronecker(matrix(sqrt(vbetaTmp) , length(betaTmp) , 1) , matrix(stdNrm , 1 , length(stdNrm)))

  return(list('c1Vec' = c1Vec , 'aVec' = aVec , 'betaMat' = betaMat))
}

##################################
### function to compute posteriors for c1, a and beta...
##################################
postc1abetaz0 <- function(c1Vec , aVec , betaMat , z , X , covsGrid , D00 = 0 , covModel = 'ExponentialC' , nu = 0.5){
  N <- length(z)
  p <- dim(X)[[2]]

#######################################    
### set default vectors for parameters if not inputted...  
#######################################    
  if (missing(c1Vec)){
    tmp <- setDefaultParamVecs(X , z)
    c1Vec <- tmp$c1Vec
  }else{}

  if (missing(aVec)){
    tmp <- setDefaultParamVecs(X , z)
    aVec <- tmp$aVec
  }else{}

  if (missing(betaMat)){
    tmp <- setDefaultParamVecs(X , z)
    betaMat <- tmp$betaMat
  }else{}

  if (p > 0){
    if (!is.matrix(betaMat)){
      betaMat <- matrix(betaMat , p , length(betaMat) / p)
    }else{} 
    nVecbeta <- dim(betaMat)[[2]]
  }else{
    nVecbeta <- 0
  }

  nc1Vec <- length(c1Vec)
  naVec <- length(aVec)
  
##############################################  
### initialize arrays and vectors for storing...  
##############################################  
  lnfpbetaa <- array(NA , c(p , nVecbeta , naVec))
  
  lnfla <- NA * numeric(naVec)
  lnfpc1a <- matrix(NA , nc1Vec , naVec)
  
### calculate prior for a... 
  if (prioraType == "reference")
  {
    lnf0a <- rep(NA, length.out = length(aVec) )
    # values will be assigned later, in the interation loop over a
  } else
  {
    lnf0a <- priora(aVec , X)
  }  

### for extra info, store each est of sigma2hatIFa and betahatIFa...  
  betahatIFa <- matrix(NA , p , naVec)
  sigma2hatIFa <- matrix(NA , 1 , naVec)
  
#######################################    
### set up for predictions, if rqd...
#######################################    
  if (missing(covsGrid)){
    calcPreds <- FALSE
  }else{
    calcPreds <- TRUE
    n0 <- dim(covsGrid)[[1]]
    X0 <- matrix(1 , n0 , p)
    if (p > 1){
      X0[,2:p] <- covsGrid
    }else{}

    if(length(D00) == 1 && D00 == 0){
      predType <- 1 # pt support, only pred vars rqd
    }else if(length(D00) > 1 && length(D00) == (nrow(X0)^2)){  
      predType <- 2 # pt support, pred covar mtx rqd
    }else{  
### for block support predictions, make DMats.a2p as , <data areas>to<prediction areas> average covariance matrices
      predType <- 3 # block support, only block pred vars rqd
    }

    z0IFa <- matrix(NA , n0 , naVec)
    if(predType == 2){
      v0IFa <- list()
    }else{
      v0IFa <- matrix(NA , n0 , naVec)
    }
  }
  
#######################################    
### loop over values of a...
#######################################    
  for (i in 1:naVec){
    txtOut <- paste0('Processing for a = ' , aVec[i] , '...'); cat("\r",txtOut)
    intLikbObj <- intLikb(aVec[i] , z = z , X = X , covModel = covModel , nu = nu)
  
### If Berger reference prior for a
## recalculate a    
    
    if (prioraType == "reference")
    {
    
#####################################################################      
#    if (DMats.a2a$method == "dp")
#    {
#      Dc <- apply(X = DMats.a2a$discPtsD, 
#                MARGIN = c(1,2), 
#                FUN = mean)
#    } else 
#      if(DMats.a2a$method == "lq")  
#    {
#        Dc <- apply(X = DMats.a2a$quadPtsD, 
#                    MARGIN = c(1,2), 
#                    FUN = mean)
#    }
#
#    lnf0a[i] <- refPriora (aVec[i], X, Dc,
#                             intLikbObj)
#####################################################################      
### ref prior fn updated, so above replaced by:
     
      lnf0a[i] <- refPriora (a = aVec[i], X = X , DMats = DMats.a2a, intLikbObj = intLikbObj , covModel = covModel , nu = nu)

    }
    

    if (p >0){
      betahatIFa[,i] <- intLikbObj$betahat
    }else{}
    sigma2hatIFa[i] <- intLikbObj$sigma2hat
    
### lnlik for a...        
    lnfla[i] <- intLikbObj$logLik
    
### for calculating posterior for c1...    
    if ((!is.infinite(lnfla[i])) && (!is.na(intLikbObj$Ac)) && (!is.na(lnf0a[i])) && (!is.infinite(lnf0a[i])) ){

      alphaTmp <- (N - p + 2 * priorc1Pars[1]) / 2
      betaTmp <- (intLikbObj$resinvAres + 2 * priorc1Pars[2]) / 2

      lnfpc1a[,i] <- alphaTmp * log(betaTmp) - lgamma(alphaTmp) - 
         (alphaTmp + 1) * log(c1Vec) - betaTmp / c1Vec + intLikbObj$logLik + lnf0a[i]

    }else{}
    
### marginal posterior for betai given this a (plus post for a)...
    if (p > 0){
      tmp <- margbetab(pars = aVec[i] , z = z , X = X , intLikbObj = intLikbObj , betaMat = betaMat)
      lnfpbetaa[,,i] <- tmp$lnmpb + as.numeric(intLikbObj$logLik) + lnf0a[i]
    }else{}
    
#####################################################    
### if rqd, compute predictions...
#####################################################    
    if (calcPreds){
      if((!is.infinite(lnf0a[i])) & (!is.na(lnf0a[i])) & (!is.infinite(lnfla[i]))  & (!is.na(lnfla[i])) ){
        tmp <- fz0IFa(z = z , X = X , X0 = X0 , pars = aVec[i] , intLikbObj = intLikbObj , D00 = D00 , covModel = covModel , nu = nu)
        
        z0IFa[,i] <- tmp$z0hat
        if (predType == 2){
          v0IFa[[i]] <- tmp$v0hat
        }else{
          v0IFa[,i] <- tmp$v0hat
        }
      }else{
        if(predType == 2){
          v0IFa[[i]] <- matrix(NA , n0 , n0)
        }else{}
      }    
    }else{}
    
  } # end of the loop over values of a.
  
#######################################################
### marginalize and normalize to give priors/liks/posteriors for c1...
#######################################################
  lnfpc1a[which(is.nan(lnfpc1a))] <- -Inf
  
### back transform...
  fpc1a <- exp(lnfpc1a - max(lnfpc1a , na.rm = T))

### marginalize by summing over a, sum(delai * av(fai , fai+1))...
  fpc1 <- rowSums(matrix(aVec[-1] - aVec[-naVec] , nc1Vec , naVec - 1 , byrow = TRUE) * 0.5 * (fpc1a[,-1] + fpc1a[,-naVec]) , na.rm = T)

### normalize...
  areaTmp <- sum((c1Vec[-1] - c1Vec[-nc1Vec]) * 0.5 * (fpc1[-1] + fpc1[-nc1Vec]) , na.rm = T)
  fpc1 <- fpc1 / areaTmp  
  
### for interest and for returning, prior for c1...
  f0c1 <- 1 / c1Vec
  f0c1[which(c1Vec <= 0)] <- NA
  
#######################################################
### just normalize to give priors/liks/posteriors for a (analytical marginaliztion already done)...
#######################################################
  lnfpa <- lnfla + lnf0a
  
### back transform...
  f0a <- exp(lnf0a - max(lnf0a , na.rm = T))
  fpa <- exp(lnfpa - max(lnfpa , na.rm = T))

### normalize (note f0a is improper, normalize just for visual comparison)...
  if (naVec > 1){

    areaTmp <- sum((aVec[-1] - aVec[-naVec]) * 0.5 * (f0a[-1] + f0a[-naVec]) , na.rm = T)
    f0a <- f0a / areaTmp

    areaTmp <- sum((aVec[-1] - aVec[-naVec]) * 0.5 * (fpa[-1] + fpa[-naVec]) , na.rm = T)
    fpa <- fpa / areaTmp

  }else{}

####################################################  
### marginal posteriors for beta...
####################################################  
  if (p > 0){
### back transform, could be done with kron, but for now going with loop...
    fpbetaa <- array(NA , dim = c(p , nVecbeta , naVec))
    for (ip in 1:p){
      fpbetaa[ip , ,] <- exp(lnfpbetaa[ip,,] - max(lnfpbetaa[ip,,] , na.rm = T))
    }
    remove(lnfpbetaa)
  
### marginalize by summing over a...
    fpbetaaTmp <- 0.5 * (fpbetaa[,,-1] + fpbetaa[,,-naVec])
    aVecTmp <- array(kronecker(aVec[-1] - aVec[-naVec] , rep(1,nVecbeta*p)) , c(p , nVecbeta , naVec - 1))
    if (p == 1)
    {
      fpbetaaTmp <- array(fpbetaaTmp, dim=c(1, dim(fpbetaaTmp) )) 
      # Force fpbetaaTmp to be an array rather than a matrix, for compatibility with aVecTmp
    }

    fpbeta <- apply(fpbetaaTmp * aVecTmp , c(1,2) , sum , na.rm = T)
    remove( fpbetaaTmp , aVecTmp) # fpbetaa ,
  
### normalize...
    areaTmp <- rowSums((betaMat[,-1,drop = FALSE] - betaMat[,-nVecbeta,drop = FALSE]) * 0.5 * (fpbeta[,-1,drop = FALSE] + fpbeta[,-nVecbeta,drop = FALSE]) , na.rm = T)
    fpbeta <- fpbeta / matrix(areaTmp , p , nVecbeta)

  }else{
    fpbeta <- NA
  }

#####################################################    
### if rqd, compute predictions...
#####################################################    
  if (calcPreds){

    aVecTmp <- matrix(aVec[-1] - aVec[-naVec] , n0 , naVec - 1 , byrow = TRUE)

    z0Tmp <- z0IFa * matrix(fpa , n0 , naVec , byrow = TRUE)
    z0Tmp <- 0.5 * (z0Tmp[,-1,drop = FALSE] + z0Tmp[,-naVec,drop = FALSE]) 
    z0 <- rowSums(z0Tmp * aVecTmp , na.rm = T)

    if (predType == 2){
      aVecTmp2 <- aVec[-1] - aVec[-naVec] 
### first part of variance, = mean of the variance...
### second part of variance, = variance of the mean...
### in this version, first sum covIFa and (muIFa-mu) %*% t(muIFa-mu), then do weighted average
      v0Tmp <- list()
      for (ia in 1:naVec){
        z0Tmp <- z0IFa[,ia,drop = FALSE] - z0
        v0Tmp[[ia]] <- v0IFa[[ia]] + z0Tmp %*% t(z0Tmp)
      }
      v0Tmp <- mapply("*" , v0Tmp , fpa , SIMPLIFY = FALSE)
      v0Tmp <- mapply("+" , v0Tmp[-1] , v0Tmp[-naVec] , SIMPLIFY = FALSE)
      v0Tmp <- lapply(v0Tmp , "*" , 0.5) 
      v0Tmp <- mapply("*" , v0Tmp , aVecTmp2 , SIMPLIFY = FALSE)
      ### delete any v0Tmp that have any NAs in...
      iOK <- unlist(lapply(v0Tmp , function(x) !any(is.na(x))))
      v0 <- Reduce('+' , v0Tmp[iOK])

    }else{
### first part of variance, = mean of the variance...
      v0Tmp <- v0IFa * matrix(fpa , n0 , naVec , byrow = TRUE)
      v0Tmp <- 0.5 * (v0Tmp[,-1,drop = FALSE] + v0Tmp[,-naVec,drop = FALSE]) 
      v0 <- rowSums(v0Tmp * aVecTmp , na.rm = T)

### second part of variance, = variance of the mean...
      v0Tmp <- (z0IFa - matrix(z0 , n0 , naVec , byrow = FALSE)) ^ 2
      v0Tmp <- v0Tmp * matrix(fpa , n0 , naVec , byrow = TRUE)
      v0Tmp <- 0.5 * (v0Tmp[,-1,drop = FALSE] + v0Tmp[,-naVec,drop = FALSE]) 

      v0 <- v0 + rowSums(v0Tmp * aVecTmp , na.rm = T)
    }
  }else{
    z0 <- NA    
    v0 <- NA    
  }  
  
  print('Posterior and predictive distributions calculated!')
  
#######################################################
### and return...
#######################################################
  return(list('c1Vec' = c1Vec , 'f0c1' = f0c1 , 'fpc1' = fpc1 ,
              'aVec' = aVec , 'f0a' = f0a , 'fpa' = fpa , 
              'betaMat' = betaMat , 'fpbeta' = fpbeta , 
              'z0' = z0 , 'v0' = v0 , 
              'betahatIFa' = betahatIFa , 'sigma2hatIFa' = sigma2hatIFa,
              'fpc1a' = fpc1a,
              'fpbetaa' = fpbetaa,
              'z0IFa'  = z0IFa,
              'v0IFa'  = v0IFa
              ))
} 

#############################################
### plot the posteriors and priors for covariance parameters...
#############################################
plotPostCovParams <- function(c1Vec , f0c1 , fpc1 , aVec , f0a , fpa , newPlot){
  if (missing(newPlot)){
    newPlot = TRUE
  }else{}
  
  if (newPlot){
    dev.new()
  }else{}
  par(mfrow = c(1,2))
  
# scale just using post
  tmp <- fpc1[!is.infinite(fpc1)]
  ymax <- 1.05 * max(tmp , na.rm = TRUE)
  plot(c1Vec , f0c1 , col = 'red' , ylim = c(0 , ymax) , type = 'l' , lwd = 3 , xlab = 'c1' , ylab = 'marginal pdf')
  lines(c1Vec , fpc1 , col = 'green' , lwd = 3)
  legend("topright" , legend = c('prior' , 'posterior') , fill = c('red' , 'green'))

# scale just using lik and post (prior might tend to inf for small vals)  
  tmp <- fpa[!is.infinite(fpa)]
  ymax <- 1.05 * max(tmp , na.rm = TRUE)
  
  plot(aVec , f0a , col = 'red' , ylim = c(0 , ymax) , type = 'l' , lwd = 3 , xlab = 'a' , ylab = 'marginal pdf' )
  lines(aVec , fpa , col = 'green' , lwd = 3)
  legend("topright" , legend = c('prior' , 'posterior') , fill = c('red' , 'green'))
}

#############################################
### plot the posteriors for trend parameters...
#############################################
plotPostTrendParams <- function(betaMat , fpbeta , newPlot){
  p <- dim(betaMat)[[1]]
  if (missing(newPlot)){
    newPlot = TRUE
  }else{}
  
  if (newPlot){
    dev.new()
  }else{}
  
  par(mfrow = c(1,p))
  
  # scale just using lik and post (prior might tend to inf for small vals)  
  for (ip in 1:p){
    ymax <- 1.05 * max(fpbeta[ip,] , na.rm = TRUE)
    
    plot(betaMat[ip,] , fpbeta[ip,] , col = 'green' , ylim = c(0 , ymax) , type = 'l' , lwd = 3 , xlab = '' , ylab = 'marginal pdf' )
    legend("topright" , legend = c(paste0('beta[' , ip , ']')) , fill = c('green'))
  }
}


fz0IFa <- function(z , X , X0 , pars , intLikbObj , z0Vec , D00 = 0 , covModel = 'ExponentialC' , nu = 0.5){
##################################################################
### send in a intLikb object with all the things calculated in that function
### function only calculates N0 predictions and N0 prediction variances (not an N0xN0 prediction covariance matrix)
### the integrated likelihood fn (intLikb) for <data and value at prediction location> zd0 is exactly equal to...
### intLikb(zd) * fz0IFzd...
##################################################################
  a <- as.numeric(pars[1])

  N <- length(z)
  p <- dim(X)[[2]]

#  variography:::setParameters(cvm) <- c(0,1,a)
#  Ad0 <- as(cvm, "function")(Dcd0)

#  Ad0 <- exp(-Dcd0 / a)
#  Ad0 <- apply(Ad0,MARGIN=c(1,2),FUN=mean)

### make DMats.a2p in main script, and get it from there.
  Ad0 <- averageCov(covModel = covModel , covPars = c(1 , a) , DMats = DMats.a2p , nu = nu)

  if(length(D00) == 1 && D00 == 0){
    A00 <- 1
    predType <- 1 # pt support, only pred vars rqd
  }else if(length(D00) > 1 && length(D00) == (nrow(X0)^2)){  
    A00 <- exp(-D00 / a)
    predType <- 2 # pt support, pred covar mtx rqd
  }else{  
### for block support predictions, make DMats.a2p as , <data areas>to<prediction areas> average covariance matrices
#    A00 <- mean(exp(-D00 / a))
    A00 <- averageCov(covModel = covModel , covPars = c(1 , a) , DMats = D00 , nu = nu)
    predType <- 3 # block support, only block pred vars rqd
  }

  invAAd0 <- intLikbObj$invA %*% Ad0
  A0dinvAz <- t(invAAd0) %*% z

  if (p > 0){  
    A0dinvAX <- t(invAAd0) %*% X
    #browser()
    X0_A0dinvAX <- X0 - A0dinvAX # again, better for all locns together.
  }else{
    A0dinvAX <- X
    X0_A0dinvAX <- c()
  }
  z0hat <- X0 %*% intLikbObj$betahat + A0dinvAz - A0dinvAX %*% intLikbObj$betahat

  if(predType == 1 || predType == 3){
### A00 should be either a scalar or a vector of lentgh n0 (not a full n0 x n0 matrix)  
    A0IFdUK <- A00 - colSums(Ad0 * invAAd0) # shortcut (using elementwise multiplication) to perform above for all pred locns at once.
    if (p > 0){
      A0IFdUK <- A0IFdUK + t(colSums(t(X0_A0dinvAX) * (intLikbObj$invXinvAX %*% t(X0_A0dinvAX)))) # elementwise multiplication as shortcut
    }else{}
  }else if (predType == 2){
### updating to make it matrix form...
    A0IFdUK <- A00 - t(Ad0) %*% invAAd0 # no shortcut 
    if (p > 0){
      A0IFdUK <- A0IFdUK + X0_A0dinvAX %*% (intLikbObj$invXinvAX %*% t(X0_A0dinvAX)) # no shortcut
    }else{}
  }else{
    stop('Invalid prediction type!')
  }


### mY and qC are from De Oliveira et al., 1997,
### equiv to t dist (Roth's vs) with mu = z0hat, Sigma = sigma2hatREML * A0IFdUK , N - p df  
#  mY <- z0hat[1]
#  qC <- (intLikbObj$resinvAres + 2 * priorc1Pars[2]) * A0IFdUK
#  
#  v0hat <- qC / (N - p + 2 * priorc1Pars[1] - 2)

### or in my notation: 
#  Sigma <- (intLikbObj$resinvAres + 2 * priorc1Pars[2]) * A0IFdUK / (N - p + 2 * priorc1Pars[1])
#  nu <- N - p + 2 * priorc1Pars[1]

  v0hat <- (intLikbObj$resinvAres + 2 * priorc1Pars[2]) * A0IFdUK / (N - p + 2 * priorc1Pars[1] - 2)
  
######################################################	
### just test the full old pdf using the first location...  
######################################################	
  # if ((length(z0Vec) == 1) && (is.na(z0Vec))){
  #   z0Vec <- seq(z0hat[1] - 4 * sqrt(A0IFdUK[1]) , z0hat[1] + 4 * sqrt(A0IFdUK[1]) , 8 * sqrt(A0IFdUK[1]) / 100)
  # }else{}
  # 
  # tmp  <- lgamma((N - p + 1) / 2) - 0.5 * log(pi) - lgamma((N - p) / 2) - 0.5 * log(qC)
  # lnfz0IFa1 <- tmp - ((N - p + 1) / 2) * log(1 + (((z0Vec - mY) ^ 2) / qC))
  # fz0IFa1 <- exp(lnfz0IFa1)
  # 
  # return(list('z0hat' = z0hat , 'v0hat' = v0hat , 'z0Vec' = z0Vec , 'fz0IFa1' = fz0IFa1))
  
######################################################	
  return(list('z0hat' = z0hat , 'v0hat' = v0hat)) 
}

######################################################	
### calculate stats of a pdf, fx (after normalizing)...  
######################################################	
pdfStats <- function(x , fx , verbose){
### assumes for now x has constant diffs...
  if (missing(verbose)){ verbose = FALSE }else{}
  
  delx <- x[2] - x[1]
  areafx <- sum(delx * fx)

  fx <- fx / areafx

  meanfx <- sum(delx * x * fx)
  varfx <- sum(delx * ((x - meanfx) ^ 2) * fx)

  if (verbose){
    print(paste0('Orig area = ' , areafx , '; mean = ' , meanfx , '; var = ' , varfx)    )
  }else{}

  return(list('fx' = fx , 'areafx' = areafx , 'meanfx' = meanfx , 'varfx' = varfx))  
}

##################################################################################
##################################################################################
##################################################################################


