calculate_validation_stats <- function(li_simulation_data,
                              vn_z_pred,
                              vn_z_pred_sd,
                              bo_remove_area_0 = FALSE,
                              bo_remove_points_with_area_0 = FALSE)
  
## Compares prediction and prediction sd with original signal; returns
#  prediction error, StSE, and summary validation statistics
{
  
  
  # vn_z_pred = li_REML_results$vn_pred 
  # vn_z_pred_sd = li_REML_results$vn_sd
  # bo_remove_area_0 = TRUE
  
  #names(li_simulation_data$df_data_points)
  
  vn_z <- li_simulation_data$df_data_points$z
  
  
  fraction_inside_region <- function(vn_z,
                                     vn_z_pred,
                                     vn_z_pred_sd,
                                     n_sd_factor)
  {
    mean( 
      vn_z > vn_z_pred - vn_z_pred_sd * n_sd_factor &
      vn_z < vn_z_pred + vn_z_pred_sd * n_sd_factor
    )
  }
  
  if ( bo_remove_points_with_area_0)
    # needed in case of baseline; remove area = 0 from the points
  {
    vn_z <- vn_z[li_simulation_data$df_data_points$area > 0]      
    
  }  
  
  vn_error <- vn_z - vn_z_pred
  
  ## Prediction validation 
  # Mean error
  n_mean_error <- mean(vn_error)
  
  # Root Mean Squared error
  n_rmse       <- sqrt( mean( vn_error^2) )
  
  ## Prediction error validation 
  # Fraction inside region
  n_1_sd_fir <- fraction_inside_region(vn_z, 
                                   vn_z_pred,
                                   vn_z_pred_sd,
                                   n_sd_factor = 1)
  
  n_2_sd_fir <- fraction_inside_region(vn_z, 
                                   vn_z_pred,
                                   vn_z_pred_sd,
                                   n_sd_factor = 2)
  
  # Standardised squared error 
  # See Lark, 2000, https://doi.org/10.1046/j.1365-2389.2000.00345.x
  
  vn_stse    <- vn_error^2/vn_z_pred_sd^2 
  
  n_mean_stse   <- mean(vn_stse)
  n_median_stse <- median(vn_stse)
  
  
  ## mass preserving property


    if (bo_remove_points_with_area_0)
    # can happen in case of baseline; remove area = 0 from the points
    {
     # browser()
      df_data_points_tmp <- li_simulation_data$df_data_points[li_simulation_data$df_data_points$area > 0, ]

      df_mass_pres_diff <- data.frame( obs   = li_simulation_data$df_data_area$mz,
                                       mean_preds = tapply(X     = vn_z_pred, 
                                                           INDEX = df_data_points_tmp$area, 
                                                           FUN   = mean)
      )

    }  
  

  if(bo_remove_area_0)
  {
  # Remove area = 0 from the area means
      df_mass_pres_diff <- data.frame( obs   = li_simulation_data$df_data_area$mz,
                                       mean_preds = tapply(X     = vn_z_pred, 
                                                           INDEX = li_simulation_data$df_data_points$area, 
                                                           FUN   = mean)[-1]
      )
      
  }
      
  if(!bo_remove_points_with_area_0 & !bo_remove_area_0)
  {
    df_mass_pres_diff <- data.frame( obs   = li_simulation_data$df_data_area$mz,
                                     mean_preds = tapply(X     = vn_z_pred, 
                                                         INDEX = li_simulation_data$df_data_points$area, 
                                                         FUN   = mean)
    ) 
    
  }
  
  n_max_mass_pres_diff <- (
    max(
      abs(df_mass_pres_diff$obs-df_mass_pres_diff$mean_preds)
    )
  )
  
  return(
    list(
      vn_error        = vn_error,
      vn_stse         = vn_stse,
      n_mean_error    = n_mean_error,
      n_rmse          = n_rmse,
      n_1_sd_fir      = n_1_sd_fir,
      n_2_sd_fir      = n_2_sd_fir,
      n_mean_stse     = n_mean_stse,
      n_median_stse   = n_median_stse,
      n_max_mass_pres_diff = n_max_mass_pres_diff
    )
    
  )

}
