calculate_validation_stats <- function(vn_original_signal,
                                       spdf_prediction#,
                                       #vn_z_pred,
                                       #vn_z_pred_sd
                                       )
{
  
  # spdf_prediction <- spdf_prediction_REML
  
  vn_z <- vn_original_signal
  vn_z_pred     <- spdf_prediction$prediction
  vn_z_pred_sd  <- spdf_prediction$prediction_sd
  
  
  fraction_inside_region <- function(vn_z,
                                     vn_z_pred,
                                     vn_z_pred_sd,
                                     n_sd_factor)
  {
    mean( 
      vn_z > vn_z_pred - vn_z_pred_sd * n_sd_factor &
        vn_z < vn_z_pred + vn_z_pred_sd * n_sd_factor
    )
  }
  
  vn_error <- vn_z - vn_z_pred
  
  ## Prediction validation 
  # Mean error
  n_mean_error <- mean(vn_error)
  
  # Root Mean Squared error
  n_rmse       <- sqrt( mean( vn_error^2) )
  
  ## Prediction error validation 
  # Fraction inside region
  n_1_sd_fir <- fraction_inside_region(vn_z, 
                                       vn_z_pred,
                                       vn_z_pred_sd,
                                       n_sd_factor = 1)
  
  n_2_sd_fir <- fraction_inside_region(vn_z, 
                                       vn_z_pred,
                                       vn_z_pred_sd,
                                       n_sd_factor = 2)
  
  # Standardised squared error 
  # See Lark, 2000, https://doi.org/10.1046/j.1365-2389.2000.00345.x
  
  vn_stse    <- vn_error^2/vn_z_pred_sd^2 
  
  n_mean_stse   <- mean(vn_stse)
  n_median_stse <- median(vn_stse)
  
  
  ## mass preserving property
  
  df_mass_pres_diff <- data.frame( obs = tapply(X     = vn_z, 
                                                       INDEX = spdf_prediction$REG_MAP_ID, 
                                                       FUN   = mean),
                                   mean_preds = tapply(X     = vn_z_pred, 
                                                       INDEX = spdf_prediction$REG_MAP_ID, 
                                                       FUN   = mean)
  )
  
  n_max_mass_pres_diff <- (
    max(
      abs(df_mass_pres_diff$obs-df_mass_pres_diff$mean_preds)
    )
  )
  
  return(
    list(
      vn_error        = vn_error,
      vn_stse         = vn_stse,
      n_mean_error    = n_mean_error,
      n_rmse          = n_rmse,
      n_1_sd_fir      = n_1_sd_fir,
      n_2_sd_fir      = n_2_sd_fir,
      n_mean_stse     = n_mean_stse,
      n_median_stse   = n_median_stse,
      n_max_mass_pres_diff = n_max_mass_pres_diff,
      df_mass_pres_diff = df_mass_pres_diff
    )
    
  )
  
}
