create_covariance_objects <- function(spdf_pred_grid_with_cov_plan,
                                      sp_pred_grid_plan, 
                                      spoldf_areas_plan = NULL 
                                      )
{

  ## Creates two covariance objects in the gobal environment:
  # - DMats.a2a area to area description 
  # - DMats.a2p area to point for prediction
  # These objects will be used later to calculate the correlation 
  # matrices for different values of phi 
  
  ## different options: 
  # on polygones & legendre-gauss quadrature (lq) - fast but probably less reliable 
  # on points & discretisation points (dp) - slow and requires lots of memory
  
    
  
  if(!is.null(spoldf_areas_plan)) # Polygones and lq
  {
    
    str_avg_cov_method <- "lq"
    
    
    DMats.a2a <<- DMats4AverageCov.a2a(areas1 = spoldf_areas_plan, 
                                       method   = str_avg_cov_method,
                                       nQuadPts = 10
    )
    
    DMats.a2p <<- 
      DMats.a2p <- DMats4AverageCov.a2p (#discPts1 = li_disc_pnts,
        points2  = sp_pred_grid_plan,
        areas1 = spoldf_areas_plan,
        method   = str_avg_cov_method,
        nQuadPts = 10
      )
    
    
  } else # on points (where cultivation takes place) and using dp
  {
    #browser()
    str_avg_cov_method <- "dp"
    
    vn_areaIds <- unique(spdf_pred_grid_with_cov_plan$REG_MAP_ID)
    
    vn_areaIds <- vn_areaIds[order(vn_areaIds)]
    
    df_ar <- as.data.frame(spdf_pred_grid_with_cov_plan@coords)
    df_ar$region <- spdf_pred_grid_with_cov_plan$REG_MAP_ID
    
    size <- 50  #former: 50
    
    nareas <- length(vn_areaIds)
    sampleregions<-NULL
    resol <- 5
    set.seed(314)
    li_disc_pnts <- list()
    for (i in 1:nareas) {
      ids <- which(df_ar$region==vn_areaIds[i])
      samids <- sample(ids,size=size,replace=T)
      reg1sam <- df_ar[samids,c("x","y")]
      reg1sam$x <- jitter(reg1sam$x,resol)
      reg1sam$y <- jitter(reg1sam$y,resol)
      reg1sam$area <- rep(vn_areaIds[i],each=length(reg1sam$x))
      sampleregions<-rbind(sampleregions,reg1sam)
      coordinates(reg1sam) <- ~x+y
      proj4string(reg1sam) <- str_prj_planar
      li_disc_pnts[i] <- reg1sam
      # for (j in 1:i) {
      #   ids <- which(df_ar$region==vn_areaIds[j])
      #   samids <- sample(ids,size=size,replace=T)
      #   reg2sam <- df_ar[samids,c("x","y")]
      #   reg2sam$x <- jitter(reg2sam$x,resol)
      #   reg2sam$y <- jitter(reg2sam$y,resol)
      #   coordinates(reg2sam) <- ~x+y
      #   proj4string(reg2sam) <- prj
      #   Dc[i,j,] <- as.vector(spDists(x=reg1sam,y=reg2sam,longlat=T))
      #   Dc[j,i,] <- Dc[i,j,]
      # }
    }
    
    
    DMats.a2a <<- DMats4AverageCov.a2a(discPts1 = li_disc_pnts,
                                       method   = str_avg_cov_method
    )
    
    DMats.a2p <<-
      DMats4AverageCov.a2p (discPts1 = li_disc_pnts,
                            points2  = sp_pred_grid_plan,
                            method   = str_avg_cov_method )
    
    
    # This code was used to check for MPP
    # length(li_disc_pnts)
    # 
    # spdf_tmp_pred_points <- li_disc_pnts[[1]]
    # 
    # for (i in 2:45)
    # {
    #   
    #   spdf_tmp_pred_points <-
    #     rbind(spdf_tmp_pred_points, li_disc_pnts[[i]] )
    #   
    # }
    # 
    # 
    # spplot(spdf_tmp_pred_points, 
    #        zcol      = "area",
    #        cex       = 0.25,
    #         scales    = list(draw = TRUE)
    # )
    # 
    # 
    # str(spdf_tmp_pred_points)
    # 
    #   DMats.a2p <<- 
    #      DMats4AverageCov.a2p (discPts1 = li_disc_pnts,
    #       points2  = spdf_tmp_pred_points,#li_disc_pnts,
    #       method   = str_avg_cov_method )
    # 
    #   
    #   dim(DMats.a2a$discPtsD)    
    #   dim(DMats.a2p$discPtsD)
    
    
  }
  gc(verbose = FALSE)  
  invisible()
}  
