load_saved_results_create_tables <- function(n_dimensionality = 1, # 1 or 2
                                             bo_open_tables = TRUE, # open tables for inspection in GUI
                                             bo_bw          = FALSE 
                                             # No coloured numbers (value dependend) in summary table
                                             )
## Reads the .RData files from the simuation ensemble, and produces the longtable body of many 
# results and the tabular part of a summary table, both in Latex format
  
## Note that this function is hard-coded to the specific purpose;
# far more flexible outputs are possible with R packages such as xtable.
  
## This  function contains the supporting functions
# format_single_element and
# add_one_session_to_table
  
{  
  

  if(n_dimensionality==1)
  {
    str_source_dir          <- "./session_results_1D_sim/"
    str_output_file         <- "./export_tables/table-1D-ensemble-output.tex"  # big table
    str_output_file_summary <- "./export_tables/table-1D-ensemble-output-summary.tex" # summary table
    vn_m <- c(10, 15, 
              20, 30, 50)
    
    vs_lists <- c("li_results10",
                  "li_results15",
                  "li_results20",
                  "li_results30",
                  "li_results50")
    
    vb_select_for_summary <- c(TRUE, FALSE, TRUE, FALSE, TRUE) 
    # Because the 1D summary table skips m = 15 and m = 30
    
    
  } else if(n_dimensionality==2)
  {
    str_source_dir          <- "./session_results_2D_sim/"    
    str_output_file         <- "./export_tables/table-2D-ensemble-output.tex"  # big table
    str_output_file_summary <- "./export_tables/table-2D-ensemble-output-summary.tex" # summary table

    vn_m <- c(9, 
              25, 
              49)
    
    vs_lists <- c("li_results09",
                  #"li_results15",
                  "li_results25",
                  #"li_results30",
                  "li_results49")
    
    vb_select_for_summary <- c(TRUE, TRUE, TRUE) 
    # Because the 2D summary table does not skip m's
    
  }


    
  format_single_element <- function(n_mean, n_sd)
  {
    
    if(!is.na(n_mean))
    {
      
      n_mean_print <- format(
        round(n_mean, digits=2), 
        nsmall = 2)
      n_sd_print <- format(
        round(n_sd, digits=2), 
        nsmall = 2)
      
      if (n_mean > 1 & !bo_bw)
      {
        str_return <- paste0("\\textcolor[rgb]{", 
                             round(min(n_mean - 1,1),  digits = 3),
                             ",0,0}{", 
                             n_mean_print, 
                             "} {\\tiny ", 
                             n_sd_print, 
                             "}")
      } else if (n_mean < 1 & !bo_bw)
      {
        str_return <- paste0("\\textcolor[rgb]{0,0,", 
                             round(min(1 - n_mean,1),  digits = 3),
                             "}{", 
                             n_mean_print, 
                             "} {\\tiny ", 
                             n_sd_print, 
                             "}")    
      }  else
      {
        str_return <- paste0(n_mean_print, 
                             " {\\tiny ", 
                             n_sd_print, 
                             "}")    
      }
      return(str_return)
    } else # n_mean is NA
    {
      return("")
    }
  } # END function format_single_element
  
  
  add_one_session_to_table <- function(str_file, 
                                       str_session_description_short, 
                                       bo_suppress_baseline = FALSE)
  {
  
    load(file=
           paste0(str_source_dir,str_file, ".RData")
      )
    
    str_summary_variable <- "meanStSE"
    
    ## number or rows for each dataframe (assumed to be equal)
    n_approaches <- 6
    
    
    n_m      <- length(vn_m)
    
    #eval(parse(text = paste0("df_to_show =  ", vs_df_to_show[1] )) )
    
     #nrow(df_to_show)
    
    str_table <- paste0(str_table, " \\multirow{", 
        n_approaches * n_m, 
        "}{\\linewidth}{ \\rot{ \\parbox{",
        1.25 * n_approaches * n_m, #height of multirow expressed in Latex "ex"       
        "ex}{", 
        str_session_description_short, 
        "}}} & " )
    
  
    n_m_summary <- sum(vb_select_for_summary)

    str_sum_table <- paste0(str_sum_table, 
                            "\\hline",
                            "\\multirow{", 
                            n_m_summary, 
                            "}{\\linewidth}{", str_session_description_short, "} & " 
                            )
    
    
    for ( ii in 1:length(vn_m) )
    {
      
      if (ii>1)
      {
        str_table <- paste0(str_table, "& ")
        if(vb_select_for_summary[ii])
        {
          str_sum_table <- paste0(str_sum_table, " & ")
        }  
      } 
      
      eval(parse(text = paste0("df_to_show_means =  ", vs_lists[ii], "$df_simulation_means" ) ) )
      eval(parse(text = paste0("df_to_show_sds =  ", vs_lists[ii], "$df_simulation_sd" ) ) )   
      
      
      
      ## Additional question: Worse than baseline, based on RMSE? ####
      # Commented out, not used in the final paper. This is however an
      # example of post-processing of the simulation results
      
      # n_simulations <- 250 #1D; for 2D, set 128
      # eval(parse(text = paste0("li_simulation =  ", vs_lists[ii], "$li_simulation" ) ) )
      # 
      # vb_baseline_better <- rep(NA, length.out = n_simulations)
      # for (iii in 1:n_simulations)
      # {
      #   vb_baseline_better[iii] <- li_simulation[[iii]]["Baseline","RMSE"]<max(li_simulation[[iii]][-6,"RMSE"], na.rm = TRUE)
      # }  
      # cat("\n Worse than baseline?", str_session_description_short, " | ", vs_lists[ii], "\n")
      # cat( mean(vb_baseline_better) )
      ## END Additional question 
      
    
      ## change order
      df_to_show_means <- df_to_show_means[c(1, 5, 3, 2, 4, 6),]
      df_to_show_sds   <- df_to_show_sds  [c(1, 5, 3, 2, 4, 6),]   
      
      df_to_show_means <- df_to_show_means[,c(1:7, 12, 8:11)]
      df_to_show_sds   <- df_to_show_sds  [,c(1:7, 12, 8:11)]
      
      if(bo_suppress_baseline)
      {
        df_to_show_means["Baseline",] <- NA
        df_to_show_sds["Baseline",]   <- NA
      }
      
      
      str_table <- paste0(str_table, "\\multirow{", n_approaches, "}{*}{", vn_m[ii] , "} " )
      
      if(vb_select_for_summary[ii])
      {
        str_sum_table <- paste0(str_sum_table, 
                         "  ", 
                         vn_m[ii]
                         )
      
          str_sum_table <- paste0(str_sum_table,
                           gsub('NA','-',
                                paste0(
                                  " & ", format_single_element(df_to_show_means["REML",str_summary_variable],
                                                               df_to_show_sds["REML",str_summary_variable]
                                                               ), 
                                  " & ", format_single_element(df_to_show_means["MML",str_summary_variable],
                                                               df_to_show_sds["MML",str_summary_variable]
                                                               ),
                                  " & ", format_single_element(df_to_show_means["InvGam",str_summary_variable],
                                                               df_to_show_sds["InvGam",str_summary_variable]
                                                               ),
                                  " & ", format_single_element(df_to_show_means["Uniform",str_summary_variable],
                                                               df_to_show_sds["Uniform",str_summary_variable]
                                                               ),                            
                                  " & ", format_single_element(df_to_show_means["Reference",str_summary_variable],
                                                               df_to_show_sds["Reference",str_summary_variable]
                                                               )
                                 
                                )
                           ), 
                           "\\\\ \n"
          )
          
         
      }  # END if(vb_select_for_summary[ii])
      
      for (i in 1:n_approaches)
      {
        if (i>1)
        {
          str_table <- paste0(str_table, "& ")
        }  
        str_table <- paste0(str_table, " & ", rownames(df_to_show_means)[i])
        str_table <- paste0(str_table,  
          gsub('NA','-',     # replace the literal 'NA'
               paste0("& ", 
                      c(
                        format(
                          df_to_show_means[i,1], 
                          scientific = TRUE,
                          digits = 3,
                          nsmall = 2),
                        format(
                          round(df_to_show_means[i,2:9], digits=2), 
                          scientific = FALSE,
                          nsmall = 2),
                        format(
                          round(df_to_show_means[i,10:12], digits=3), 
                          nsmall = 3)                                        
                      ),
                      " {\\tiny ", 
                      gsub('NA','',
                           c(
                             format(
                               df_to_show_sds[i,1], 
                               scientific = TRUE,
                               digits = 3,
                               nsmall = 2),
                             format(
                               round(df_to_show_sds[i,2], digits=2), 
                               nsmall = 2),                           
                             format(
                               round(df_to_show_sds[i,3:7], digits=2), 
                               nsmall = 2),
                             format(
                               round(df_to_show_sds[i,8:9], digits=1), 
                               nsmall = 1),
                             "",                           
                             "",
                             ""
                           )
                        ),
                      "}"  , 
                      collapse = ""
                    ) 
          )
        )
        
        str_table <- paste0(str_table, "\\\\ \n")
   
        
        
      } ## END for (i in 1:n_approaches)
      
      str_table <- paste0(str_table, "\\cline{2-", n_columns, "}")

      if(vb_select_for_summary[ii])
      {
        if (i < n_approaches)
        {
          str_sum_table <- paste0(str_sum_table,
                                  "\\cline{2-7} \n")

        }
      }

    } ## END for(ii in 1:length(vn_m))
    
    str_table <<- paste0(str_table , "\\hline \n")
    str_sum_table <<- str_sum_table   
    
    
  } # END add_one_session_to_table
  
  
  #### Start with Latex tables ####
  ## The large output table 
  str_table <- "% Note: this is only the body part of the longtable \n"
  n_columns <- 15
  
  ## The summary table
  #str_summary_variable <- "n_mean_stse"
  n_columns_summary_table <- 8
  str_sum_table <- "
  % Note: this is only the tabular part of the table
  \n
  \\begin{tabular}{p{3.5cm} "  # string for summary table
  
  str_sum_table <- paste0(str_sum_table,
                   paste0(rep("c ", times = n_columns_summary_table-1) , collapse = ''),
                   "}\n",
                   "Session & m & {REML} & {MML} & {InvGam} & {Uniform} & {Reference} ",
                   "\\\\ \n"
                   )
  
  
  ## For the Latex coder' convenience, here is the rest of the longtable for the 1D simulations,
  # commented out
  
  # \captionsetup{width=\linewidth}
  # \begin{longtable}{|p{1cm}|c|l|c| c| c| c| c| c| c| c| c|c|c|c|}
  # \caption*{Table. Results one-dimensional simulation ensemble with validating on original signal. Shown results are the average over, or the fraction of, 250 simulations. Smallfont numbers show the corresponding standard deviation. 
  #   \newline
  #   $m$: number of area means, c.q. number of observations;
  #   \textit{Algorithm}: explained in the text;
  #   \textit{ME}: mean error;     
  #   \textit{RMSE}: root mean squared error;
  #   \textit{FIR(1) or FIR(2)}: fraction of original signal inside region of +/- 1 or +/- 2 prediction sd;    
  #   \textit{StSE}: standardised squared error;     
  #   \textit{max(MPP)}: the maximal found difference with respect to the mass preserving property; 
  #   $\hat \sigma^2$: estimated variance according to REML; 
  #   $mode(\sigma^2)$ mode of the variance according to the Bayesian approaches
  #   $\hat \phi$: estimated range parameter according to REML and MML; 
  #   $mode(\phi)$ mode of the range parameter according to the  Bayesian approaches;
  #   \textit{$\phi_{sim}$ in C.I.}: $\phi_{sim}$ inside 90\% credibility interval of posterior $\phi$;
  #   \textit{$\hat \phi$, $mode(\phi)$ approx min, max} : $\hat \phi$ (REML, MML) or mode $\phi$ (Bayesian) very close to  respectively minimal or maximal possible value.\\
  #   Note that  $\hat \phi$ of MML equals $mode(\phi)$ of Bayesian uniform by definition. 
  # } 
  # \label{tb:results-simulation-ensemble}  \\
  # \hline
  # & & & 
  #   \multicolumn{9}{|c|}{ \textit{Average over simulations}}&
  #   \multicolumn{3}{|c|}{ \textit{Fraction of simulations} }
  # \\ %\hline
  # Session & $m$ & Algorithm & ME & RMSE & FIR(1) & FIR(2) & mean(StSE)& med(StSE)& \rot{max(MPP)} & \rot {$\hat \sigma^2$, $mode(\sigma^2)$} & \rot{$\hat \phi$, $mode(\phi)$} & \rot{$\phi_{sim}$ in C.I.} &  
  #   \rot { $\hat \phi$, $mode(\phi) \ approx ~ min$ } &
  #   \rot { $\hat \phi$, $mode(\phi) \ approx ~ max$ }
  # \\ \hline
  # \endfirsthead 
  # \multicolumn{4}{c}%
  # %{\tablename\ \thetable\ -- \textit{Continued from previous page}} \\
  # { \textit{Continued from previous page}} \\
  # \hline
  # \rot{Session} & $m$ & Algorithm & ME & RMSE & FIR(1) & FIR(2) & mean(StSE)& med(StSE)& \rot{max(MPP)} & \rot {$\hat \sigma^2$, $mode(\sigma^2)$} & \rot{$\hat \phi$, $mode(\phi)$} & \rot{$\phi_{sim}$ in C.I.} &  
  #   \rot { $\hat \phi$, $mode(\phi) \approx ~ min$ } &
  #   \rot { $\hat \phi$, $mode(\phi) \approx ~ max$ }
  # \\ \hline
  # \endhead
  # \hline \multicolumn{4}{r}{\textit{Continued on next page}} \\
  # \endfoot
  # \hline
  # \endlastfoot
  # 
  # INSERT HERE THE EXPORT OF str_table
  #
  #  \end{longtable}
  
   
  
  #### Fill Latex tables ####
  

      add_one_session_to_table("Session_01",
                               str_session_description = "1: Standard settings")
      
      add_one_session_to_table("Session_02",
                               str_session_description = "2: Small upper limit: $\\phi_u = 100$",
                               bo_suppress_baseline = TRUE)
      
      if(n_dimensionality == 1)
        str_table <- paste0(str_table, "\\pagebreak \n")
  
      
      add_one_session_to_table("Session_03",
                               str_session_description = "3: Large upper limit: $\\phi_u = 2,000$",
                               bo_suppress_baseline = TRUE)
      
      if(n_dimensionality == 2)
        str_table <- paste0(str_table, "\\pagebreak \n")      
      
      add_one_session_to_table("Session_04",
                               str_session_description = "4: Trend is Gaussian Random field")
      
      if(n_dimensionality == 1)
        str_table <- paste0(str_table, "\\pagebreak \n")
      
      add_one_session_to_table("Session_05",
                               str_session_description = "5: No trend")
      
      add_one_session_to_table("Session_06",
                               str_session_description = "6: Mat\\'ern simulation, smooth; $\\nu_{sim} = 2$")
      
      if(n_dimensionality == 1)
        str_table <- paste0(str_table, "\\pagebreak \n")
      
      add_one_session_to_table("Session_07",
                               str_session_description = "7: Mat\\'ern simulation, unsmooth; $\\nu_{sim} = .25$")
      
      if(n_dimensionality == 2)
        str_table <- paste0(str_table, "\\pagebreak \n")      
      
      add_one_session_to_table("Session_08",
                               str_session_description = "8: Added nugget: $\\tau^2 = 1$")
      
      if(n_dimensionality == 1)
        str_table <- paste0(str_table, "\\pagebreak \n")
      
      add_one_session_to_table("Session_09",
                               str_session_description = "9: Extreme short range in simulation: $\\phi_{sim} = 5$")
      
      add_one_session_to_table("Session_10",
                               str_session_description = "10: Extreme long range in simulation: $\\phi_{sim} = 600$")
      
  #### Finish & write Latex tables ####
  
  str_table <- paste0(str_table, "
  \\hline \n
  %\\end{longtable}\n ")
  
  
  str_sum_table <- paste0(str_sum_table,"
  \\hline \n
  \\end{tabular}\n
   ")
  
  
  write(str_sum_table, file = str_output_file_summary)
  write(str_table, file = str_output_file) 
   
  
  if(bo_open_tables)
  {
    file.edit(str_output_file)
    file.edit(str_output_file_summary)
  }

}  
















