# Please see README.pdf 

## Performs an ensemble of 1D simulations and the successive validation;
# provides Table 3 in paper, and the table in Online Resource C.

## 

## Note that this script is quite calculation intensive
# With 2018 equipment, using a 32-core computing 
# facility, it run for a few days

#### Packages and basic functions module ####
vs_needed_packages <- c("sp", # oa. for averageCov.R
                        "invgamma", # for dinvgamma in function priora postParams.R, in case you want a inv gamma prior
                        "DEoptim", # for function estimate_and_predict_REML
                        #"xtable",  # for nice output of tables
                        "parallel", # for iterative simulations
                        "statmod"  # averageCov.R
)


source("setup.R")
setup(vs_needed_packages)
remove(vs_needed_packages)

#### General  settings ####

li_general_settings <- list()
li_general_settings$n_dimensionality     <- 1  # One dimensional simulations
li_general_settings$bo_covariate         <- TRUE

#### General  settings simulation####
li_general_settings$li_simulation_generate$n_of_au             <- 300  #length of 1D simulation, in number of abstract units
li_general_settings$li_simulation_generate$n_of_nodes          <- 600 
# Number of nodes in 1D simulation. In this script, 
# this equals the number of prediction points  
li_general_settings$li_simulation_generate$n_of_observed_areas <- 30 # Number of 'observed' area means in 1D simulation 
li_general_settings$li_simulation_generate$bo_random_areas     <- FALSE # if FALSE, areas will be regulary


li_general_settings$li_simulation_generate$bo_trend_on_coord   <- TRUE
li_general_settings$li_simulation_generate$n_beta_1            <- 0    # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 0.02  # Regression coefficient ()

li_general_settings$li_simulation_generate$str_covModel        <-  "ExponentialC"
li_general_settings$li_simulation_generate$n_nu                <- NA # Smoothnessparameter of matern function

li_general_settings$li_simulation_generate$n_c0                <- 0   # Nugget default
li_general_settings$li_simulation_generate$n_c1                <- 5   # Partial sill
li_general_settings$li_simulation_generate$n_a                 <- 60  # Range parameter /distance parameter /distance parameter
li_general_settings$li_simulation_generate$n_grid_density      <- 350 
# choice of grid density over total research area, 
# for calculating mean covariance between and within areas, and between
# area and prediction points



## Settings that uniform, REML, MME

li_general_settings$li_multiple_simulations$n_runs <- 250  

source("run_multiple_simulations.R")

li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

source("save_1D_session.R")

save_1D_session(str_file = "Session_01", 
             str_session_description = "1: Standard"
)

## Session 2

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)

li_general_settings$lu_bounds <- c(10, 100) 
li_general_settings$bo_skip_gamma_reference <- TRUE

li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_02", 
             str_session_description = "2: lower upper bound"
)


## Session 3: high upper bound ####

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)

li_general_settings$lu_bounds <- c(10, 2000) 
li_general_settings$bo_skip_gamma_reference <- TRUE

li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_03", 
             str_session_description = "3: high upper bound"
)


## Session 4 Trend is Gaussian ####

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)

li_general_settings$lu_bounds <- c(10, 300) 
li_general_settings$bo_skip_gamma_reference <- FALSE


li_general_settings$li_simulation_generate$bo_trend_on_coord   <- FALSE
li_general_settings$li_simulation_generate$n_c0_trend          <- 0    # Nugget default trend
li_general_settings$li_simulation_generate$n_c1_trend          <- 5  # Partial sill trend
li_general_settings$li_simulation_generate$n_a_trend           <- 30   # 20 Range parameter trend
li_general_settings$li_simulation_generate$n_beta_1            <- 0    # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 1  # Regression coefficient ()



li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_04", 
             str_session_description = "4: Trend is Gaussian"
)


## Session 5 No trend ####

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)

li_general_settings$lu_bounds <- c(10, 300) 
li_general_settings$bo_skip_gamma_reference <- FALSE

li_general_settings$li_simulation_generate$bo_trend_on_coord   <- TRUE
li_general_settings$li_simulation_generate$n_c0_trend          <- NA   # Nugget default trend
li_general_settings$li_simulation_generate$n_c1_trend          <- NA   # Partial sill trend
li_general_settings$li_simulation_generate$n_a_trend           <- NA   # 20 Range parameter trend
li_general_settings$li_simulation_generate$n_beta_1            <- 0    # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 0    # Regression coefficient ()

li_general_settings$bo_covariate <- FALSE

li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_05", 
             str_session_description = "5: No trend"
)


## Session 6 Matern, smooth  ####

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)


li_general_settings$li_simulation_generate$n_beta_2            <- 0.02  # Regression coefficient ()
li_general_settings$bo_covariate <- TRUE

li_general_settings$li_simulation_generate$str_covModel       <-  "MaternC" #"ExponentialC" # # or "ExponentialC"
li_general_settings$li_simulation_generate$n_nu               <- 2 #0.5  # Smoothnessparameter of matern function




li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_06", 
             str_session_description = "6: Matern smooth"
)


## Session 7 Matern, unsmooth  ####

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)



li_general_settings$li_simulation_generate$n_nu               <- .25 #0.5  # Smoothnessparameter of matern function

li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_07", 
             str_session_description = "7: Matern unsmooth"
)


## Session 8 nugget  ####

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)




li_general_settings$li_simulation_generate$str_covModel       <-  "ExponentialC" 
li_general_settings$li_simulation_generate$n_c0               <- 1   # Nugget default
li_general_settings$li_simulation_generate$n_nu               <- NA 




li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_08", 
             str_session_description = "8: Matern unsmooth"
)



## Session 9 short sim range   ####

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)


li_general_settings$li_simulation_generate$n_c0               <- 0   # Nugget default

li_general_settings$li_simulation_generate$n_a                 <- 5   #60   # Range parameter /distance parameter


li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_09", 
             str_session_description = "9: Short sim range"
)


## Session 10 short sim range   ####

remove( li_results10, 
        li_results15,
        li_results20, 
        li_results30, 
        li_results50)


li_general_settings$li_simulation_generate$n_a                 <- 600   #60   # Range parameter /distance parameter


li_general_settings$li_simulation_generate$n_of_observed_areas <- 10
li_results10 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 15
li_results15 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 20
li_results20 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$n_of_observed_areas <- 30
li_results30 <- run_multiple_simulations(li_general_settings)


li_general_settings$li_simulation_generate$n_of_observed_areas <- 50
li_results50 <- run_multiple_simulations(li_general_settings)

save_1D_session(str_file = "Session_10", 
             str_session_description = "10: Long sim range"
)



li_results10$df_simulation_summary
li_results15$df_simulation_summary
li_results20$df_simulation_summary
li_results30$df_simulation_summary
li_results50$df_simulation_summary



################################

##### Create emsemble results tables #####
# The previously saved session results are loaded and
# processed into Table 3 and Online Resource C
source("load_saved_results_create_tables.R")
load_saved_results_create_tables(n_dimensionality = 1, # 1 or 2
                                bo_open_tables = FALSE, # open tables for inspection in GUI
                                bo_bw  = FALSE  # No coloured numbers (value dependend) in summary table
                                )


