# Please see README.pdf 

## Creates one 1D simulation; provides
# figure 2 to 5, and table 2 in paper

#### General  settings ####

li_general_settings <- list()
#li_general_settings$bo_covariate       <- TRUE 

#### Packages and basic functions  ####
vs_needed_packages <- c("sp",       # Among others needed in create_covariance_objects_1D_sim
                        "invgamma", # for dinvgamma in function priora postParams.R, in case of inv gamma prior
                        "DEoptim",  # Optimalisation used in REML_core
                        "xtable",   # for nice output of tables in show_validation_stats
                        "statmod",  # for averageCov.R
                        "plotrix",  # for show_parameter_assessments
                        "gstat",    # used for variogram plot
                        "geoR",     # used for variogram plot
                        "scales"    # used for variogram plot
)
source("setup.R")
setup(vs_needed_packages) # If needed, packages are installed; packages are loaded
remove(vs_needed_packages, setup)


#### General  settings simulation####
li_general_settings$li_simulation_generate$n_of_au             <- 300  #length of 1D simulation, in number of abstract units
li_general_settings$li_simulation_generate$n_of_nodes          <- 600 
# Number of nodes in 1D simulation. In this script,this equals the number of prediction points  

li_general_settings$li_simulation_generate$n_of_observed_areas <- 10 # Number of 'observed' area means in 1D simulation 
li_general_settings$li_simulation_generate$bo_random_areas     <- FALSE # if FALSE, areas will be regulary
## Not used:
# li_general_settings$li_simulation_generate$n_minimal_area_size <- 50  # Minimal number of nodes in each area; only used if bo_random_areas == TRUE


li_general_settings$li_simulation_generate$bo_trend_on_coord   <- TRUE
li_general_settings$li_simulation_generate$n_beta_1            <- 0    # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 0.02  # Regression coefficient ()


li_general_settings$li_simulation_generate$str_covModel       <-  "ExponentialC" #"MaternC" # or "ExponentialC"
li_general_settings$li_simulation_generate$n_nu                <- NA #0.5  # Smoothnessparameter of matern function


li_general_settings$li_simulation_generate$n_c0                <- 0    # Nugget default
li_general_settings$li_simulation_generate$n_c1                <- 5    # Partial sill
li_general_settings$li_simulation_generate$n_a                 <- 60   # Range parameter /distance parameter
li_general_settings$li_simulation_generate$n_grid_density      <- 350  #  standard: 350
# choice of grid density over total research area, 
# for calculating mean covariance between and within areas, and between
# area and prediction points

set.seed(271828 + 26)
source("prepare_1D_simulation_data.R")
li_simulation_data <- prepare_1D_simulation_data(li_general_settings$li_simulation_generate)
li_general_settings$lu_bounds <- c(10, 300)  # Lower and upper bounds for REML and uniform prior, also affecting MML


#### Create covariance objects ####
source("create_covariance_objects_1D_sim.R")
create_covariance_objects_1D_sim(li_simulation_data)  # note: created objects are in global scope

df_data_area            <- li_simulation_data$df_data_area
df_data_points          <- li_simulation_data$df_data_points 


lm_bestmodel <- lm(mz ~ mx, data = df_data_area)
mn_x0 <- cbind(1, df_data_points$x)


#### REML estimation and prediction ####
li_general_settings$li_simulation_assess_REML$vn_a_bounds          <- c(0.25, 300) #c(0.5,300)
li_general_settings$li_simulation_assess_REML$n_max_iter           <- 20


source( "REML_core.R") 
set.seed(271828) # REML optimalisation uses some randomness
li_REML_results <- REML_core(lm_bestmodel = lm_bestmodel,
                             vn_zbar      = df_data_area$mz,
                             li_settings_REML = li_general_settings$li_simulation_assess_REML,
                             #mn_initial_pop = as.matrix(seq(50,200, length.out = 20), nrow=1, ncol=20),
                             mn_x0 = mn_x0
)

source("calculate_validation_stats.R")


li_val_stats_REML <- 
  calculate_validation_stats(
    li_simulation_data = li_simulation_data,
    vn_z_pred = li_REML_results$vn_pred ,
    vn_z_pred_sd = li_REML_results$vn_sd
  )   

c(
  unlist(li_val_stats_REML[3:9])
)


li_REML_results$n_a   # phi = range parameter
li_REML_results$n_c1  # sigmasq = partial sill

#### Bayesian inference and prediction ####

priorc1Pars      <<-  c(0, 0)
c1Vec <-  seq(1e-5, 20 , length.out = 200)

Xc   <- cbind(1,
              df_data_area$mx)
ma_covsGrid <- matrix(df_data_points$x, ncol = 1)


#### Uniform ####
prioraType      <<- 'uni'
prioraPars      <<- c(0.1, 300.4)

#prioraPars      <<- c(li_general_settings$lu_bounds[1]-0.05,  li_general_settings$lu_bounds[2]+ 0.05)

aVec  <- c(
  seq(0.25 ,  2.25 , 0.2),
  seq(3 ,  50 , 1),
  seq(55,  300 , 5)
)




betaMat <- rbind(
  seq(-6, 6, length.out = 100) ,
  seq(-0.02, 0.06, length.out = 100)          
)


li_BAK_results <- postc1abetaz0(z = c(df_data_area$mz) , 
                                X = Xc , 
                                aVec = aVec , 
                                c1Vec = c1Vec , 
                                betaMat = betaMat,
                                covsGrid =  ma_covsGrid
)

n_a <- li_BAK_results$aVec[li_BAK_results$fpa == max(li_BAK_results$fpa)]



#### Show simulation data and results ####

## General settings export tables and plots 
li_general_settings$li_export$bo_plots_to_latex  <- TRUE # as .pdf

bo_bw <- TRUE    # black-white or colour  

if (bo_bw)
{  
  li_general_settings$li_export$latex_plot_dir     <-
    "./exportbw/"
} else
{
  li_general_settings$li_export$latex_plot_dir     <-
    "./exportcolour/"
}  
li_general_settings$li_export$use_greek     <- TRUE
# If TRUE: use in some plots phi and sigma^2 rather than a and c1

li_general_settings$li_export$bo_tables_to_latex  <- TRUE # as .tex
li_general_settings$li_export$str_tables_to_latex <- "./export_tables/"

source("show_1D_simulation_data_and_prediction_results.R")
# Produces figure 2 in paper
show_1D_simulation_data_and_prediction_results(li_simulation_data, 
                                            li_estimations_and_predictions_REML = li_REML_results,
                                            li_estimations_and_predictions_BAK  = li_BAK_results,
                                            li_export                           = li_general_settings$li_export,
                                            bo_bw                               = bo_bw
)


#### Show parameter assessments ####
# Produces figures 3 and 4 in paper
source("show_parameter_assessments.R")
#file.edit("show_parameter_assessments.R")

li_general_settings$li_export$use_greek     <- TRUE  # use phi and sigmasq rather than a and c1

show_parameter_assessments(li_BAK_results, 
                           li_REML_results,
                           li_general_settings$li_simulation_generate,
                           li_export = li_general_settings$li_export,
                           bo_show_marginals_postParams  = FALSE, # the build in function in postParams
                           bo_show_marginals  = FALSE, # plots adjusted for simulated case                                        
                           bo_show_joint_c1_a = TRUE,
                           bo_show_joint_beta_a = TRUE,
                           bo_bw              =  bo_bw
                           )
#

####### Create variogram ######
# Among others, produces Figure 5
source("show_variogram.R")  
show_variogram(li_export = li_general_settings$li_export,
               li_simulation_data,
               li_BAK_results,
               li_REML_results,
               bo_bw = bo_bw)

## For testing purposes
# system("open ./exportbw/variogram-reml-density.pdf")
# system("open ./exportcolour/variogram-reml-density.pdf")



#### For comparison: Baseline prediction ####

vn_z_pred_baseline <- as.numeric(
  li_simulation_data$df_data_area$mz[li_simulation_data$df_data_points$area]
)


#### Calculate and export validation stats ####
## Table 2 in paper

source("calculate_validation_stats.R")

li_val_stats_REML <- 
  calculate_validation_stats(
    li_simulation_data = li_simulation_data,
    vn_z_pred = li_REML_results$vn_pred,
    vn_z_pred_sd = li_REML_results$vn_sd
  ) 

li_val_stats_BAK  <- 
  calculate_validation_stats(
    li_simulation_data = li_simulation_data,
    vn_z_pred = li_BAK_results$z0,
    vn_z_pred_sd = sqrt(li_BAK_results$v0)
  )                         


li_val_stats_baseline  <- 
  calculate_validation_stats(
    li_simulation_data = li_simulation_data,
    vn_z_pred = vn_z_pred_baseline,
    vn_z_pred_sd = NA
  )    



source("show_validation_stats_1D_sim.R")
## Returns summary validation statistics, eventually exports a latex table
# (table 2 in paper). Plots the prediction residuals, the prediction residuals density,
# the QQ-plot of the residuals and standardised squared prediction 
# error density; not used in paper. 
df_validation_results <- 
  show_validation_stats_1D_sim(li_val_stats_BAK,
                         li_val_stats_REML,
                         li_val_stats_baseline,
                         bo_show_plots  = TRUE, 
                         li_export = li_general_settings$li_export
  )

