# Please see README.pdf 

## Performs an ensemble of 2D simulations and the successive validation;
# provides Table D-1 in Online Resource D and the table in Online Resource E

## 

## Note that this script is quite calculation intensive
# With 2018 equipment, using a 32-core computing facility, it run for several days



#### Packages and basic functions module ####
vs_needed_packages <- c("sp",       # oa. for averageCov.R
                        "invgamma", # for dinvgamma in function priora postParams.R, in case you want a inv gamma prior
                        "DEoptim",  # for function estimate_and_predict_REML
                        "parallel", # 
                        "statmod",  # averageCov.R
                        "geoR",     # for simulation
                        "RandomFields" # For 2D simulations
)

source("setup.R")
setup(vs_needed_packages)
remove(vs_needed_packages)

#### General  settings ####

li_general_settings <- list()
li_general_settings$n_dimensionality     <- 2  # Two dimensional simulations
li_general_settings$bo_covariate         <- TRUE
li_general_settings$li_export$bo_show_plots <- FALSE

#### General  settings simulation ####
# size of grid
li_general_settings$li_simulation_generate$vn_size_grid        <- c(300, 300)  #length x width of 2D simulation, in number of abstract units
li_general_settings$li_simulation_generate$vn_of_nodes         <- c(100, 100)
li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 

li_general_settings$li_simulation_generate$bo_trend_on_coord   <- TRUE
li_general_settings$li_simulation_generate$n_beta_1            <- 0     # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 0.02  # Regression coefficient ()

li_general_settings$li_simulation_generate$str_covModel <-  "MaternC" # or "ExponentialC"
li_general_settings$li_simulation_generate$n_nu         <- 0.5  # Smoothnessparameter of matern function

li_general_settings$li_simulation_generate$n_c0 <- 0   # Nugget 
li_general_settings$li_simulation_generate$n_c1 <- 5   # Partial sill
li_general_settings$li_simulation_generate$n_a  <- 60  # Range parameter /distance parameter /distance parameter  

## Define functions

li_general_settings$lu_bounds <- c(10, 300)  # Bounds for REML and uniform prior, also affecting MML

li_general_settings$bo_skip_gamma_reference <- FALSE


#### Parallel run many simulations #### 

source("run_multiple_simulations.R")
li_general_settings$li_multiple_simulations$n_runs <- 128 # 4*32; runs on 32 core machine

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

source("save_2D_session.R")

save_2D_session(str_file = "Session_01", 
             str_session_description = "1: Standard"
)


#### Session 2: lower upper bound

remove( li_results09, 
        li_results25,
        li_results49)

li_general_settings$lu_bounds <- c(10, 100) 
li_general_settings$bo_skip_gamma_reference <- TRUE

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_02", 
             str_session_description = "2: Lower upper bound"
)

#### Session 3: higher upper bound

remove( li_results09, 
        li_results25,
        li_results49)
li_general_settings$lu_bounds <- c(10, 2000) 
li_general_settings$bo_skip_gamma_reference <- TRUE

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_03", 
             str_session_description = "3: Higher upper bound"
)



### Session 4: Trend is Gaussian Random field ####

remove( li_results09, 
        li_results25,
        li_results49)

li_general_settings$lu_bounds <- c(10, 300) 
li_general_settings$bo_skip_gamma_reference <- FALSE

li_general_settings$li_simulation_generate$bo_trend_on_coord   <- FALSE
li_general_settings$li_simulation_generate$n_c0_trend          <- 0    # Nugget default trend
li_general_settings$li_simulation_generate$n_c1_trend          <- 5    # Partial sill trend
li_general_settings$li_simulation_generate$n_a_trend           <- 30   # 20 Range parameter trend
li_general_settings$li_simulation_generate$n_beta_1            <- 0    # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 1    # Regression coefficient ()


li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_04", 
             str_session_description = "4: Trend is GRF"
)



### Session 5: No trend ####

remove( li_results09, 
        li_results25,
        li_results49)

li_general_settings$lu_bounds <- c(10, 300) 
li_general_settings$bo_skip_gamma_reference <- FALSE

li_general_settings$li_simulation_generate$bo_trend_on_coord   <- TRUE
li_general_settings$li_simulation_generate$n_beta_1            <- 0    # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 0    # Regression coefficient ()

li_general_settings$bo_covariate <- FALSE

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_05", 
             str_session_description = "5: No trend"
)

### Session 6: Matern, smooth nu_sim = 2####

remove( li_results09, 
        li_results25,
        li_results49)

li_general_settings$li_simulation_generate$n_beta_2            <- 0.02  # Regression coefficient ()
li_general_settings$bo_covariate <- TRUE

li_general_settings$li_simulation_generate$str_covModel       <-  "MaternC" #"ExponentialC" # # or "ExponentialC"
li_general_settings$li_simulation_generate$n_nu               <- 2 #0.5  # Smoothnessparameter of matern function

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_06", 
             str_session_description = "6: Matern smooth, nu = 2"
)


### Session 7: Matern, unsmooth nu_sim = 0.25 ####

remove( li_results09, 
        li_results25,
        li_results49)

li_general_settings$li_simulation_generate$n_nu               <- .25 #0.5  # Smoothnessparameter of matern function

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_07", 
             str_session_description = "7: Matern unsmooth, nu = 0.25"
)

### Session 8: Added nugget, tau^2 = 1 ####

remove( li_results09, 
        li_results25,
        li_results49)

li_general_settings$li_simulation_generate$n_nu               <- .5 #0.5  # Smoothnessparameter of matern function
li_general_settings$li_simulation_generate$n_c0               <- 1

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_08", 
             str_session_description = "8: With nugget"
)

### Session 9: Extreme short range in simulation: phi_sim = 5 ###
remove( li_results09, 
        li_results25,
        li_results49)

li_general_settings$li_simulation_generate$n_c0               <- 0

li_general_settings$li_simulation_generate$n_a                <- 5

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_09", 
             str_session_description = "9: Extreme short range"
)



### Session 10: Extreme long range in simulation: phi_sim = 600 ###
remove( li_results09, 
        li_results25,
        li_results49)

li_general_settings$li_simulation_generate$n_a                <- 600

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 
li_results09 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(5,5) # Number of 'observed' area means in 2D simulation 
li_results25 <- run_multiple_simulations(li_general_settings)

li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(7,7) # Number of 'observed' area means in 2D simulation 
li_results49 <- run_multiple_simulations(li_general_settings)

save_2D_session(str_file = "Session_10", 
             str_session_description = "10: Extreme long range"
)


##### Create emsemble results tables #####
# The previously saved session results are loaded and
# processed into Table D-1 in Online Resource D and
# the table in Online Resource E
source("load_saved_results_create_tables.R")
load_saved_results_create_tables(n_dimensionality = 2, # 1 or 2
                                 bo_open_tables = FALSE, # open tables for inspection in GUI
                                 bo_bw  = FALSE  # No coloured numbers (value dependend) in summary table
)