# Please see README.pdf 

## Creates one 1D simulation and performs the inference, prediction and validation; 
# provides figure all figures in Online Resource D

#### General  settings ####

li_general_settings <- list()

#li_general_settings$str_path_to_postParams  <- "../postParams/"
#li_general_settings$str_path_to_shared_functions <- "../simulations-case-shared-functions/"

#### Packages and basic functions module ####

vs_needed_packages <- c("sp",       # oa. for averageCov.R
                        "invgamma", # for dinvgamma in function priora postParams.R, in case you want a inv gamma prior
                        "DEoptim",  # for function estimate_and_predict_REML
                        "statmod",  # for averageCov.R
                        "geoR",     # for simulation
                        "scales",   # for visualisation
                        "viridis",   # for visualisation
                        "RandomFields", # For 2D simulations
                        "plotrix"   # for color.legend
                  )
source("setup.R")
setup(vs_needed_packages)
remove(vs_needed_packages)



#### General  settings simulation####
# size of grid
li_general_settings$li_simulation_generate$vn_size_grid        <- c(300, 300)  #length x width of 2D simulation, in number of abstract units
li_general_settings$li_simulation_generate$vn_of_nodes         <- c(60, 60)
li_general_settings$li_simulation_generate$vn_of_observed_areas <- c(3,3) # Number of 'observed' area means in 2D simulation 



li_general_settings$li_simulation_generate$bo_trend_on_coord   <- TRUE
li_general_settings$li_simulation_generate$n_beta_1            <- 0     # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 0.02  # Regression coefficient ()


li_general_settings$li_simulation_generate$str_covModel       <-  "MaternC" # or "ExponentialC"
li_general_settings$li_simulation_generate$n_nu                <- 0.5  # Smoothnessparameter of matern function

li_general_settings$li_simulation_generate$n_c0 <- 0   # Nugget 
li_general_settings$li_simulation_generate$n_c1 <- 5   # Partial sill
li_general_settings$li_simulation_generate$n_a  <- 60  # 

li_general_settings$li_export$bo_show_plots      <- FALSE # Show or export plots
li_general_settings$li_export$latex_plot_dir     <- ""
#"M:\\My Documents\\Thesis\\latex\\A2P\\a2p-pictures\\"
if (li_general_settings$li_export$latex_plot_dir != "" )
{
  li_general_settings$li_export$bo_plots_to_latex  <- TRUE
  li_general_settings$li_export$bo_show_plots      <- TRUE
} else   
{
  li_general_settings$li_export$bo_plots_to_latex  <- FALSE
}

#### Generate signal, means, and polygons
set.seed(158) # 
source("prepare_2D_simulation_data.R")
li_generated <- prepare_2D_simulation_data(li_general_settings)


df_area_means       <- li_generated$df_area_means
df_simulation_field <- li_generated$df_simulation_field
sppoldf_area_means  <- li_generated$sppoldf_area_means

#### Resume so far ####
## So now we have:
# df_simulation_field : The complete simulated field
# df_area_means : the area means and covariate means as dataframe
# sppoldf_area_means : area means as polygones


li_general_settings$bo_covariate <- TRUE

## Prediction points equal generated points

sp_pred_grid <- df_simulation_field[,
                                    c("x", "y", "area")]; coordinates(sp_pred_grid) <- ~x+y

## For discretisation
sp_disc_grid <- df_simulation_field[df_simulation_field$area>0,
                                    c("x", "y", "area")]; coordinates(sp_disc_grid) <- ~x+y


names(sp_pred_grid) <- "REG_MAP_ID"
names(sp_disc_grid) <- "REG_MAP_ID"

str_prj_planar <- ""

## Creates DMats.a2a and DMats.a2p in global scope
source("create_covariance_objects.R")
create_covariance_objects(spdf_pred_grid_with_cov_plan = sp_disc_grid, 
                          sp_pred_grid_plan = sp_pred_grid )





#DMats.a2p$discPtsD[1:2, 1:5, 1:5]

li_general_settings$li_case_assess_PAB_uni$aVec <- 
  c( seq(0.1 , 50 , length.out = 100), seq(55 , 300 , length.out = 100) )  
li_general_settings$li_case_assess_PAB_uni$prioraPars <- c(0.1, 300)
li_general_settings$li_case_assess_PAB_uni$prioraType <- 'uni'



prioraType  <<- li_general_settings$li_case_assess_PAB_uni$prioraType
prioraPars  <<- li_general_settings$li_case_assess_PAB_uni$prioraPars
priorc1Pars <<- c(0,0)

li_PAB_results     <- postc1abetaz0(z        = df_area_means$area_mean, # mean values (observations); vector
                                    X        = cbind(1, df_area_means$area_covariate_mean), # the design matrix belonging with the observations; matrix
                                    aVec     = li_general_settings$li_case_assess_PAB_uni$aVec, # vector of a/phi values 
                                    c1Vec    = seq(0.005 , 200 , 5),                  # vector of c1/sigma^2 values
                                    #betaMat  = betaMat,       # Default will be taken
                                    ## number_of_covs_including_mean x number of desired points for assessing beta posterior matrix
                                    covsGrid = as.matrix(df_simulation_field$covariate, ncol=1) # matrix for prediction, one column for each covariate.
)

# plot(li_PAB_results$c1Vec, li_PAB_results$fpc1)
# plot(li_PAB_results$aVec, li_PAB_results$fpa)

dim(li_PAB_results$betaMat)

df_simulation_field$prediction <- li_PAB_results$z0
df_simulation_field$sd <- sqrt(li_PAB_results$v0)

## NOT USED, but perhaps useful   
#   plotPostTrendParams(li_PAB_results$betaMat , 
#                       li_PAB_results$fpbeta,
#                       newPlot  = FALSE)
#   
#   plotPostCovParams(li_PAB_results$c1Vec , 
#                     li_PAB_results$f0c1 , 
#                     li_PAB_results$fpc1 , 
#                     li_PAB_results$aVec , 
#                     li_PAB_results$f0a , 
#                     li_PAB_results$fpa,
#                     newPlot  = FALSE)
# }





n_signal_min <- floor(
  min(df_simulation_field$prediction, df_simulation_field$signal)
)
n_signal_max <- ceiling(
  max(df_simulation_field$prediction, df_simulation_field$signal)
)
n_cex_scale <- 1


#### Plot signal and areas ####

li_general_settings$li_export$str_latex_plot_dir <- "./exportcolour/"

li_general_settings$li_export$bo_plots_to_latex  <- TRUE


str_latex_plot_dir <- li_general_settings$li_export$str_latex_plot_dir
bo_plots_to_latex  <- li_general_settings$li_export$bo_plots_to_latex  
bo_show_plots <- TRUE

n_width <- 7
n_hight <- 7


if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-signal-and-area.pdf"),
      width  = n_width,
      height = n_hight
  )
}

clip(0,300, 0, 300)
plot(df_simulation_field$x, 
     df_simulation_field$y,
     cex = 0.5,
     pch = 16,
     xlim = c(0,300),
     ylim = c(0,300),
     xlab = "",
     ylab = "",
     asp = 1,
     col = viridis(100)[
       scales::rescale(df_simulation_field$signal, 
                       from = c(n_signal_min, n_signal_max),
                       to   = c(1,99)
       )]
)

mtext("Easting", 
      side = 1, 
      line = 2 ) 

mtext("Northing", 
      side = 2, 
      line = 2 )   



### Divide into rectangular areas; also create polygones for later use

vn_of_observed_areas <- li_general_settings$li_simulation_generate$vn_of_observed_areas

vn_x_area_borders <- round(
  seq(from = min(df_simulation_field$x), 
      to = max(df_simulation_field$x),
      length.out = vn_of_observed_areas [1] + 1
  )
)

vn_y_area_borders <- round(
  seq(from = min(df_simulation_field$y), 
      to = max(df_simulation_field$y),
      length.out = vn_of_observed_areas[2] + 1
  )
)

clip(-2,302, -2, 302)
abline(v = vn_x_area_borders, col = "white", lwd = 8)
abline(h = vn_y_area_borders, col = "white", lwd = 8)    
abline(v = vn_x_area_borders, col = "orange", lwd = 3)
abline(h = vn_y_area_borders, col = "orange", lwd = 3)



if(bo_plots_to_latex)
{
  dev.off()
}





#### Plot area means ####


if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-area-means.pdf"),
      width  = n_width,
      height = n_hight
  )
}



clip(0,300, 0, 300)
plot(df_simulation_field$x, 
     df_simulation_field$y,
     cex = 0.5,
     pch = 16,
     xlim = c(0,300),
     ylim = c(0,300),
     xlab = "",
     ylab = "",
     asp = 1,
     col = NA
)

mtext("Easting", 
      side = 1, 
      line = 2 ) 

mtext("Northing", 
      side = 2, 
      line = 2 )   




df_borders <-   expand.grid(vn_x_area_borders, 
                            vn_y_area_borders)

vn_area_id <- vector( mode="integer", length = nrow(df_simulation_field) )

li_polygons <- list()

n_area_id <- 1

for (i in 1:vn_of_observed_areas[1])
{
  for ( j in 1:vn_of_observed_areas[2])
  {
    n_low_limit_x <- vn_x_area_borders[i]
    n_upp_limit_x <- vn_x_area_borders[i+1]
    
    n_low_limit_y <- vn_y_area_borders[j]
    n_upp_limit_y <- vn_y_area_borders[j+1]
    
    vn_area_id <- 
      ifelse(df_simulation_field$x > n_low_limit_x & 
               df_simulation_field$x < n_upp_limit_x &
               df_simulation_field$y > n_low_limit_y & 
               df_simulation_field$y < n_upp_limit_y,
             n_area_id,
             vn_area_id 
      )
    # note that we, for simplicity, do not condsider locations on the area borders
    
    if(bo_show_plots & FALSE)
    {
      # x = mean( c(n_low_limit_x, n_upp_limit_x) ),
      # y = mean( c(n_low_limit_y, n_upp_limit_y) ),
      
      text( x = n_low_limit_x,
            y = n_low_limit_y,
            adj = c(-0.25,-0.25),
            labels = n_area_id,
            cex = 2,
            col = "orange"
      )           
    }
    
    
    pol <-                      Polygon(rbind(c(n_low_limit_x, n_low_limit_y),
                                              c(n_low_limit_x, n_upp_limit_y),
                                              c(n_upp_limit_x, n_upp_limit_y),
                                              c(n_upp_limit_x, n_low_limit_y)
    )
    )
    
    li_polygons[[n_area_id]] <-     Polygons(list(pol), n_area_id)
    
    n_area_id <- 1 + n_area_id
  }
  
}


## Calculate area means ####

df_area_means <- aggregate(x = df_simulation_field$signal,
                           by = list(df_simulation_field$area),
                           FUN = mean
)

df_area_mean_covariate <- aggregate(x = df_simulation_field$covariate,
                                    by = list(df_simulation_field$area),
                                    FUN = mean
)

df_area_means$area_covariate_mean <- df_area_mean_covariate$x

df_area_means <- df_area_means[-1,] 
# remove group zero, or: the mean of the not assigned points

names(df_area_means) <- c("area_id", "area_mean", "area_covariate_mean")


#### Create polygone object ####

sppol <- SpatialPolygons(li_polygons
)

row.names(df_area_means) <- df_area_means$area_id
sppoldf_area_means <- SpatialPolygonsDataFrame( sppol, 
                                                data = df_area_means 
)




### Just as quick and dirty check: plot means over area
n_area_id <- 1
for (i in 1:vn_of_observed_areas[1])
{
  for ( j in 1:vn_of_observed_areas[2])
  {
    n_low_limit_x <- vn_x_area_borders[i]
    n_upp_limit_x <- vn_x_area_borders[i+1]
    
    n_low_limit_y <- vn_y_area_borders[j]
    n_upp_limit_y <- vn_y_area_borders[j+1]
    
    # note that we, for simplicity, do not consider locations on the area borders
    
    
    # points( x = mean( c(n_low_limit_x, n_upp_limit_x) ),
    #         y = mean( c(n_low_limit_y, n_upp_limit_y) ),
    #         cex = 25,
    #         pch = 22,
    #         col = "white",
    #         bg = viridis(100)[
    #           scales::rescale(df_area_means$area_mean[n_area_id], 
    #                           from = range(df_simulation_field$signal),
    #                           to   = c(1,99)
    #           )]
    # )     
    
    rect(xleft     = n_low_limit_x, 
         ybottom   = n_low_limit_y, 
         xright    = n_upp_limit_x, 
         ytop      = n_upp_limit_y, 
         density = NULL, 
         angle = 45,
         col =  viridis(100)[
           scales::rescale(df_area_means$area_mean[n_area_id], 
                           from = range(df_simulation_field$signal),
                           to   = c(1,99)
           )],
         border = NA
         )
    
    # cat("\n", rescale(df_area_means$area_mean[n_area_id], 
    #                   from = range(df_simulation_field$signal),
    #                   to   = c(1,99)
    # )
    #)
    
    n_area_id <- 1 + n_area_id
  }
  
}

clip(-2,302, -2, 302)
abline(v = vn_x_area_borders, col = "white", lwd = 8)
abline(h = vn_y_area_borders, col = "white", lwd = 8)    
abline(v = vn_x_area_borders, col = "white", lwd = 3)
abline(h = vn_y_area_borders, col = "white", lwd = 3)


if(bo_plots_to_latex)
{
  dev.off()
}

#### Plot prediction ####

if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-prediction.pdf"),
      width  = n_width,
      height = n_hight
  )
}  

plot(df_simulation_field$x, 
     df_simulation_field$y,
     cex = 1.5,
     pch = 16,
     xlab = "",
     ylab = "",
     asp = 1,
     col = viridis(100)[
       scales::rescale(df_simulation_field$prediction, 
                       from = c(n_signal_min, n_signal_max),
                       to   = c(1,99)
       )]
)

mtext("Easting", 
      side = 1, 
      line = 2 ) 

mtext("Northing", 
      side = 2, 
      line = 2 )   

if(bo_plots_to_latex)
{
  dev.off()
}



#### Plot prediction error ####  


df_simulation_field$prediction_error <- df_simulation_field$signal - df_simulation_field$prediction


n_error_min <- floor(
  min(df_simulation_field$prediction_error)
)
n_error_max <- ceiling(
  max(df_simulation_field$prediction_error)
)

if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-prediction-error.pdf"),
      width  = n_width,
      height = n_hight
  )
}  



plot(df_simulation_field$x, 
     df_simulation_field$y,
     cex = 0.5,
     pch = 16,
     xlab = "",
     ylab = "",
     asp = 1,
     col = rainbow(100)[
       scales::rescale(df_simulation_field$prediction_error, 
                       from = c(n_error_min, n_error_max),
                       to=c(1,99)
       )]
)

mtext("Easting", 
      side = 1, 
      line = 2 ) 

mtext("Northing", 
      side = 2, 
      line = 2 )   






if(FALSE) # Mean per area
  ### Just as quick and dirty check: plot mean of error over area
{
  
  clip(-2,302, -2, 302)
  abline(v = vn_x_area_borders, col = "white", lwd = 8)
  abline(h = vn_y_area_borders, col = "white", lwd = 8)    
  abline(v = vn_x_area_borders, col = "orange", lwd = 3)
  abline(h = vn_y_area_borders, col = "orange", lwd = 3)
  
  df_error_area_means <- aggregate(x = df_simulation_field$prediction_error,
                                   by = list(df_simulation_field$area),
                                   FUN = mean
  )
  
  
  df_error_area_means  <- df_error_area_means[-1,] 
  
  names(df_error_area_means) <- c("area_id", "error")
  
  
  n_area_id <- 1
  for (i in 1:vn_of_observed_areas[1])
  {
    for ( j in 1:vn_of_observed_areas[2])
    {
      n_low_limit_x <- vn_x_area_borders[i]
      n_upp_limit_x <- vn_x_area_borders[i+1]
      
      n_low_limit_y <- vn_y_area_borders[j]
      n_upp_limit_y <- vn_y_area_borders[j+1]
      
      # note that we, for simplicity, do not consider locations on the area borders
      
      
      points( x = mean( c(n_low_limit_x, n_upp_limit_x) ),
              y = mean( c(n_low_limit_y, n_upp_limit_y) ),
              cex = 5,
              pch = 22,
              col = "white",
              bg = magma(100)[
                scales::rescale(df_error_area_means$error[n_area_id], 
                                from = c(n_sd_min, n_sd_max),
                                to   = c(1,99)
                )]
      )     
      
      # cat("\n", rescale(df_area_means$area_mean[n_area_id], 
      #                   from = range(df_simulation_field$signal),
      #                   to   = c(1,99)
      # )
      #)
      
      n_area_id <- 1 + n_area_id
    }
    
  }  
}  





if(bo_plots_to_latex)
{
  dev.off()
}




#### Plot prediction SD ####


n_sd_min <- floor(
  min(df_simulation_field$sd)
)
n_sd_max <- ceiling(
  max(df_simulation_field$sd)
)  

if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-prediction-sd.pdf"),
      width  = n_width,
      height = n_hight
  )
}  

# n_sd_min <- floor(
#   min(df_simulation_field$sd)
# )
# n_sd_max <- ceiling(
#   max(df_simulation_field$sd)
# )



plot(df_simulation_field$x, 
     df_simulation_field$y,
     cex = 0.71,
     pch = 16,
     xlab = "",
     ylab = "",
     asp = 1,
     col = magma(100)[
       scales::rescale( (df_simulation_field$sd)^1, 
                        from = c(n_sd_min, n_sd_max),
                        to=c(1,99)
       )]
)

mtext("Easting", 
      side = 1, 
      line = 2 ) 

mtext("Northing", 
      side = 2, 
      line = 2 )   


if(bo_plots_to_latex)
{
  dev.off()
}




#### Plot StSE ####  
df_simulation_field$StSE <- df_simulation_field$prediction_error^2 /   df_simulation_field$sd^2


if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-StSE-density.pdf"),
      width  = n_width,
      height = n_hight
  )
}   
plot(
  density(df_simulation_field$StSE,
          from = 0),
  main = "")

n_mean_StSE <- mean(df_simulation_field$StSE)

abline(v = n_mean_StSE, col = "orange")


n_ycoord <- (par("usr")[4]-par("usr")[3])*3/4

text(x = n_mean_StSE, 
     y = n_ycoord,
     pos = 4,
     labels = paste0("Mean = ", 
                     round(n_mean_StSE, digits=3)
     )
)

grid()


if(bo_plots_to_latex)
{
  dev.off()
}  





if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-StSE.pdf"),
      width  = n_width,
      height = n_hight
  )
}  

n_StSE_min <- floor(
  min(df_simulation_field$StSE)
)
n_StSE_max <- ceiling(
  max(df_simulation_field$StSE)
) 
if(n_StSE_max == 13)
  n_StSE_max <- 14


plot(df_simulation_field$x, 
     df_simulation_field$y,
     cex = 0.5,
     pch = 16,
     xlab = "",
     ylab = "",
     asp = 1,
     col = terrain.colors(100)[
       scales::rescale( df_simulation_field$StSE, 
                        from = c(n_StSE_min, n_StSE_max),
                        to=c(1,99)
       )]
)



mtext("Easting", 
      side = 1, 
      line = 2 ) 

mtext("Northing", 
      side = 2, 
      line = 2 )   


if(bo_plots_to_latex)
{
  dev.off()
}


#### Legend variable of interest ####

if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-legend-voi.pdf"),
      width  = n_width,
      height = n_hight
  )
}  

plot.new()

vs_legend <- seq(n_signal_min,n_signal_max, by=0.5)
vs_legend <- ifelse(
  rep(c(FALSE, TRUE, TRUE, TRUE), length.out=length(vs_legend)),
  NA, 
  vs_legend)
#vs_legend


color.legend(xl = 0,#320,
             yb = 0,#50,
             xr = 0.2,#370,
             yt = 1,#200,
             legend = vs_legend,
             rect.col = viridis(length(vs_legend)),
             gradient="y",
             cex = n_cex_scale)


if(bo_plots_to_latex)
{
  dev.off()
}



#### Legend prediction error ####

if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-legend-error.pdf"),
      width  = n_width,
      height = n_hight
  )
}  

plot.new()

vs_legend <- seq(n_error_min,n_error_max, length.out = 49)
vs_legend <- ifelse(
  rep(c(FALSE, TRUE, TRUE, TRUE, TRUE, TRUE), length.out=length(vs_legend)),
  NA, 
  vs_legend)
#vs_legend

color.legend(xl = 0,#320,
             yb = 0,#50,
             xr = 0.2,#370,
             yt = 1,#200,
             legend = vs_legend,
             rect.col = rainbow(length(vs_legend)),
             gradient="y",
             cex = n_cex_scale)

if(bo_plots_to_latex)
{
  dev.off()
}


#### Legend prediction sd ####

if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-legend-sd.pdf"),
      width  = n_width,
      height = n_hight
  )
}  

plot.new()

vs_legend <- seq(n_sd_min,n_sd_max, by=0.05)
vs_legend <- ifelse(
  rep(c(FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE), length.out=length(vs_legend)),
  NA, 
  vs_legend)
#vs_legend

color.legend(xl = 0,#320,
             yb = 0,#50,
             xr = 0.2,#370,
             yt = 1,#200,
             legend = vs_legend,
             rect.col = magma(length(vs_legend)),
             gradient="y",
             cex = n_cex_scale)

if(bo_plots_to_latex)
{
  dev.off()
}

#### Legend StSE ####

if(bo_plots_to_latex)
{  
  pdf(file = paste0(li_general_settings$li_export$str_latex_plot_dir, "a2p-2dsim-legend-StSE.pdf"),
      width  = n_width,
      height = n_hight
  )
}  

plot.new()

vs_legend <- seq(n_StSE_min, n_StSE_max, length.out = 14*4+1)
vs_legend <- ifelse(
  rep(c(FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE), length.out=length(vs_legend)),
  NA, 
  vs_legend)
#vs_legend[53] <- 13
#vs_legend

color.legend(xl = 0,#320,
             yb = 0,#50,
             xr = 0.2,#370,
             yt = 1,#200,
             legend = vs_legend,
             rect.col = terrain.colors(length(vs_legend)),
             gradient="y",
             cex = n_cex_scale)

if(bo_plots_to_latex)
{
  dev.off()
}
