## Plots Figure B1 in Online Resource B:
# example reference priors with different sample design

# Note that we complete two full runs, including prediction, just to get the reference prior plot.
# This is much easier in coding, but includes a lot of needless computations indeed.



vs_needed_packages <- c("sp")
source("setup.R")
setup(vs_needed_packages)
remove(vs_needed_packages)
source("./spatial_covariance.R") # needed for simulation


#### Figure 1:  Reference priors for 30 and 10 areas ####

li_general_settings <- list()
#### General  settings simulation####
li_general_settings$li_simulation_generate$n_of_au             <- 300  #length of 1D simulation, in number of abstract units
li_general_settings$li_simulation_generate$n_of_nodes          <- 6000 
# Number of nodes in 1D simulation. In this script, 
# this equals the number of prediction points  
li_general_settings$li_simulation_generate$n_of_observed_areas <- 30 # Number of 'observed' area means in 1D simulation 
li_general_settings$li_simulation_generate$bo_random_areas     <- FALSE # if FALSE, areas will be regulary
li_general_settings$li_simulation_generate$bo_trend_on_coord   <- TRUE
#li_general_settings$li_simulation_generate$n_minimal_area_size <- 50    
# Minimal number of nodes in each area; only used if bo_random_areas == TRUE
li_general_settings$li_simulation_generate$n_beta_1            <- 0    # Intercept
li_general_settings$li_simulation_generate$n_beta_2            <- 0.1#0.02  # Regression coefficient (regression on distance)
li_general_settings$li_simulation_generate$n_c0                <- 0    # Nugget default
li_general_settings$li_simulation_generate$n_c1                <- 5    # Partial sill
li_general_settings$li_simulation_generate$n_a                 <- 60   # 20 Range parameter
li_general_settings$li_simulation_generate$n_grid_density      <- 600  #200  
# choice of grid density over total research area, 
# for calculating mean covariance between and within areas, and between
# area and prediction points



#### 1 ####

set.seed(271828 + 64 + 16)

source("prepare_1D_simulation_data.R")
li_simulation_data <- prepare_1D_simulation_data(li_general_settings$li_simulation_generate)


# li_general_settings$li_simulation_assess_BAK$str_avg_cov_method   <- 'dp'
# li_general_settings$li_simulation_assess_BAK$c1Vec                <- seq(1 ,   10 , 0.1) 
# parameter grid c1



source("create_covariance_objects_1D_sim.R")
create_covariance_objects_1D_sim(li_simulation_data)  # note: created objects are in global scope

df_data_area            <- li_simulation_data$df_data_area
df_data_points          <- li_simulation_data$df_data_points 

priorc1Pars      <<-  c(0, 0)
c1Vec <-  c(1,2)

betaMat <- rbind(
  seq(-6, 6, length.out = 3) ,
  seq(-0.02, 0.06, length.out = 3)          
)


Xc   <- cbind(1,
              df_data_area$mx)

prioraType      <<- 'reference'
aVec                 <- c(
  seq(0  ,  2 , .05),
  #seq(2.125 ,  5 , .125),
  seq(6,  100 , 1)
)



#ma_covsGrid <- NA # No need for prediction

ma_covsGrid <- matrix(df_data_points$x, ncol = 1)

# lm_bestmodel <- lm(mz ~ mx, data = df_data_area)
# mn_x0 <- cbind(1, df_data_points$x)

li_posteriors_and_predictions_BAK <- postc1abetaz0(
                                z = c(df_data_area$mz) , 
                                X = Xc , 
                                aVec = aVec , 
                                c1Vec = c1Vec , 
                                betaMat = betaMat,
                                covsGrid =  ma_covsGrid
)



#### Number 2 ####

# parameter grid a; has large influence on calculating time
li_general_settings$li_simulation_assess_BAK$betaVec_standardised <- seq(-6 ,   6, 0.1) 
# Standardised settings for parametergrid for each beta; the final grid is
# calculated given the covariate values and the observations
# li_general_settings$li_simulation_assess_BAK$prioraType           <- 'uni'
# li_general_settings$li_simulation_assess_BAK$prioraPars           <-  c(0, 100.5)

li_general_settings$li_simulation_assess_BAK$prioraType           <- 'reference'
# li_general_settings$li_simulation_assess_BAK$prioraPars           <-  NA

# prioraType = 'uni', 'invgamma' or "reference"
# for uni specify lb and ub
# for invgamma specify alpha (shape) and beta (scale) 
li_general_settings$li_simulation_assess_BAK$priorc1Pars          <-  c(0, 0)
# prior for c1 is invgamma (no need to specify)
# priorc1Pars = c(0 , 0): prop to 1 / c1
# priorc1Pars = c(-1 , 0): uni for c1


source("estimate_and_predict_BAK.R")
#file.edit("estimate_and_predict_BAK.R")

#li_general_settings$li_simulation_assess_BAK$aVec                 <- seq(1 ,  50, 0.5)# parameter grid a



li_posteriors_and_predictions_BAK <- #li_postParams
  estimate_and_predict_BAK(li_simulation_data, 
                           li_general_settings$li_simulation_assess_BAK,
                           str_avg_cov_method = "dp"
  )







li_general_settings$li_simulation_generate$n_of_observed_areas <- 10 # Number of 'observed' area means in 1D simulation 
set.seed(271828 + 64+16)

li_simulation_data <- prepare_1D_simulation_data(li_general_settings$li_simulation_generate)

create_covariance_objects_1D_sim(li_simulation_data) 

df_data_area            <- li_simulation_data$df_data_area

Xc   <- cbind(1,
              df_data_area$mx)

li_posteriors_and_predictions_BAK_2 <- postc1abetaz0(
  z = c(df_data_area$mz) , 
  X = Xc , 
  aVec = aVec , 
  c1Vec = c1Vec , 
  betaMat = betaMat,
  covsGrid =  ma_covsGrid
)


# li_posteriors_and_predictions_BAK_2 <- #li_postParams
#   estimate_and_predict_BAK(li_simulation_data, 
#                            li_general_settings$li_simulation_assess_BAK,
#                            str_avg_cov_method = "dp"
#   )
# 





#### Plot reference prior ####
li_general_settings$li_export$bo_plots_to_latex  <- FALSE # as .pdf
li_general_settings$li_export$latex_plot_dir     <- "../plots/"
li_general_settings$li_export$use_greek     <- TRUE

source("show_reference_prior.R")

show_reference_prior(li_posteriors_and_predictions_BAK  = li_posteriors_and_predictions_BAK,
                     str_context = "simulations",
                     li_posteriors_and_predictions_BAK_2  = li_posteriors_and_predictions_BAK_2,
                     li_export                           = li_general_settings$li_export,
                     str_alternative_file_name          = "ref-prior-simulation.pdf")
