## Plots Figure B2 in Online Resource B:
# example reference prior with and without covariates

# Note that we complete two full runs, including thinned prediction, just to get the reference prior plot.
# This is much easier in coding, but indeed includes some needless computations.


#### Packages and basic functions module ####

vs_needed_packages <- c("sp",
                        "raster",
                        "rgdal")
source("setup.R")
setup(vs_needed_packages)
remove(vs_needed_packages)

li_general_settings <- list()

## Choose here variable of interest (voi)
li_general_settings$str_case  <- "PAR"   ## Synthetic case data
#li_general_settings$str_case  <- "sorghum" 
#li_general_settings$str_case  <- "millet" 

#li_general_settings$str_path_to_postParams  <- "./postParams/" OBSOLETE
#li_general_settings$str_path_to_shared_functions <- "../simulations-case-shared-functions/"
li_general_settings$str_path_to_obs_data    <- "./data_burkina_faso/"


#### Load observation data Burkina Faso ####
source("load_case_data.R")

li_obs_data_cov_predgrid <- load_case_data(li_general_settings,
                                                   n_prediction_thinning = 50
                                                   # for example 50 for testing purposes; speeds up enormously
                                                   # set 1 for production purposes
                                                   # >= 100 gives problems, disappearing provinces!
)  


#### Reference priors Sorghum ( 4 covariates) ####

li_general_settings$str_case  <- "sorghum"

df_data_area            <- data.frame(
  li_obs_data_cov_predgrid$df_yields$sorghum
)
names(df_data_area) <- "yield"

#vector with means of response variable for regions
vn_zbar <- df_data_area$yield


spdf_pred_grid_with_cov <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov        

Xc <- as.matrix(
  data.frame(intercept       = seq(from = 1, to=1, length.out = 45), 
             CFAPAR_27_scd   = NA,
             CRAIN_EC_27_scd = NA, 
             TMIN_EC_21_scd  = NA,
             PHAQ10_scd      = NA)
)


# Fill the design matrix with the relevant covariates
for (i in c("CFAPAR_27_scd", "CRAIN_EC_27_scd", "TMIN_EC_21_scd", "PHAQ10_scd"))
{  
  
  Xc[,i] <- tapply(X = spdf_pred_grid_with_cov@data[,i],
                   INDEX = spdf_pred_grid_with_cov$REG_MAP_ID,
                   FUN = mean,
                   na.rm = TRUE
  )
  
}  
remove(i)

df_voi_and_covars <- data.frame(
  cbind(vn_zbar, Xc)
)
# 'voi': variable of interest, in this case sorghum yield

names(df_voi_and_covars)[1] <- "voi"


lm_bestmodel <- lm(voi ~ CFAPAR_27_scd+CRAIN_EC_27_scd+TMIN_EC_21_scd+PHAQ10_scd, 
                   data = df_voi_and_covars
)

spdf_pred_grid_with_cov <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov
spoldf_areas            <- li_obs_data_cov_predgrid$spoldf_areas

## covariance matrix for predictions
mn_x0 <- cbind(1, as.matrix(li_obs_data_cov_predgrid$spdf_pred_grid_with_cov@data[, c(6, 8,10,12)]) )  

mn_x0_BAK <- mn_x0[,2:5] # The BAK algorithm expects a differently shaped covariate prediction matrix    


#### Give all spatial objects a planar projection ####
str_prj_planar <- "+proj=robin +datum=WGS84"
spdf_pred_grid_with_cov_plan <- spTransform(li_obs_data_cov_predgrid$spdf_pred_grid_with_cov, CRS(str_prj_planar) )
spoldf_areas_plan <- spTransform(li_obs_data_cov_predgrid$spoldf_areas, CRS(str_prj_planar) )
sp_pred_grid_plan <- spTransform(li_obs_data_cov_predgrid$sp_pred_grid, CRS(str_prj_planar) )



#### Create covariance objects ####
## Create DMats.a2a and DMats.a2p objects in the general scope, 
## for later use  for average covariance calculations 
## This takes a few seconds
source("create_covariance_objects.R")
create_covariance_objects(spdf_pred_grid_with_cov_plan, 
                          sp_pred_grid_plan 
                          #,spoldf_areas_plan
)


priorc1Pars <<- c(0,0)

k <- dim(Xc)[[2]]


Xc <- model.matrix(lm_bestmodel)  # BAK needs a column of ones


prioraType           <<- 'reference'
prioraPars           <<-  NA

aVec <- 
  c( 10, 
     100, 
     seq(500 , 50E3 , length.out = 40) , 
     seq(50E3 , 100E3 , length.out = 15) )



li_posteriors_and_predictions_BAK <- postc1abetaz0(z        = vn_zbar, 
                                    X        = Xc , 
                                    aVec     = aVec, 
                                    # c1Vec    = c1Vec, # Default
                                    #betaMat  = betaMat, # Default
                                    covsGrid = mn_x0_BAK
)



#### Millet: same areas, no covariates

# We chance only the covariance matrices, because z has no influence on the prior

Xc <- as.matrix(
  data.frame( intercept       = seq(from = 1, to=1, length.out = 45) )
)
mn_x0_BAK <- matrix(NA, nrow = length(li_obs_data_cov_predgrid$spdf_pred_grid_with_cov), ncol = 1)  

li_posteriors_and_predictions_BAK_2 <- postc1abetaz0(z        = vn_zbar, 
                                                   X        = Xc , 
                                                   aVec     = aVec, 
                                                   # c1Vec    = c1Vec, # Default
                                                   #betaMat  = betaMat, # Default
                                                   covsGrid = mn_x0_BAK
)



#########################################



#### Plot reference prior ####
li_general_settings$li_export$bo_plots_to_latex  <- TRUE # as .pdf
li_general_settings$li_export$latex_plot_dir     <- "./exportcolour/"
li_general_settings$li_export$use_greek     <- TRUE

source("show_reference_prior.R")

show_reference_prior(li_posteriors_and_predictions_BAK  = li_posteriors_and_predictions_BAK,
                     str_context = "case",
                     li_posteriors_and_predictions_BAK_2  = li_posteriors_and_predictions_BAK_2,
                     li_export                           = li_general_settings$li_export,
                     str_alternative_file_name          = "ref-prior-case.pdf")
