prepare_2D_simulation_data <- function(li_general_settings)
{
  
  bo_plots_to_latex <- li_general_settings$li_export$bo_plots_to_latex 
  bo_show_plots     <- li_general_settings$li_export$bo_show_plots
  latex_plot_dir    <- li_general_settings$li_export$latex_plot_dir
  
  n_tausq   <- li_general_settings$li_simulation_generate$n_c0    # Nugget 
  n_sigmasq <- li_general_settings$li_simulation_generate$n_c1    # Partial sill
  n_phi     <- li_general_settings$li_simulation_generate$n_a     # Range (parameter)
  
  n_nu      <- li_general_settings$li_simulation_generate$n_nu    # Matern smoothness parameter
  
  ## Default model
  RM_model <- RMmatern(nu  = 0.5, var=n_sigmasq, scale=n_phi) + 
              RMnugget(var = n_tausq) 

  if (li_general_settings$li_simulation_generate$str_covModel == "ExponentialC")
  {
    RM_model <- RMexp(var = n_sigmasq, scale = n_phi) + 
                RMnugget(var = n_tausq) 
  } 
    
  if (li_general_settings$li_simulation_generate$str_covModel == "MaternC")
  {
    RM_model <- RMmatern(nu  = n_nu, var = n_sigmasq, scale = n_phi) + 
                RMnugget(var = n_tausq) 
  }
  
  vn_size_grid <- li_general_settings$li_simulation_generate$vn_size_grid 
  
  df_simulation_field <- NULL
  
  if(is.null(df_simulation_field))
  {
    
    vn_of_nodes <- li_general_settings$li_simulation_generate$vn_of_nodes
    
    ## Basic grid
    df_simulation_field <- expand.grid(
      seq(from = 0, to = vn_size_grid[1], by = vn_size_grid[1]/vn_of_nodes[1] ),
      seq(from = 0, to = vn_size_grid[2], by = vn_size_grid[2]/vn_of_nodes[2] )
    )
  }  
  
  names(df_simulation_field) <- c("x","y")
  
  
  RFoptions(seed=NA, spConform=FALSE)
  
  vn_simulated_spatial <- RFsimulate(model = RM_model, 
                                     x = df_simulation_field$x, 
                                     y = df_simulation_field$y, 
                                     n=1)
  

  ## trend on x + y coordinate
  n_beta_1 <- li_general_settings$li_simulation_generate$n_beta_1
  n_beta_2 <- li_general_settings$li_simulation_generate$n_beta_2 
    
  if (li_general_settings$li_simulation_generate$bo_trend_on_coord)
  {

      df_simulation_field$covariate <- df_simulation_field$x + df_simulation_field$y
      vn_mean <- n_beta_1 + n_beta_2*df_simulation_field$covariate
  } else # trend is gaussian random field
  {
    n_c0_trend <- li_general_settings$li_simulation_generate$n_c0_trend   # Nugget default trend
    n_c1_trend <- li_general_settings$li_simulation_generate$n_c1_trend   # Partial sill trend
    n_a_trend  <- li_general_settings$li_simulation_generate$n_a_trend    # Range parameter trend
    
    RM_model_trend <- RMexp(var = n_c1_trend, scale = n_a_trend) + 
                      RMnugget(var = n_c0_trend) 
    
    RFoptions(seed = as.integer(
                      round(
                        runif(1, min = 1, max = 1e8) 
                        )
                      ),
              spConform=FALSE)
    
    vn_simulated_spatial_trend <- RFsimulate(model = RM_model_trend , 
                                         x = df_simulation_field$x, 
                                         y = df_simulation_field$y, 
                                         n=1)
    
    df_simulation_field$covariate <- vn_simulated_spatial_trend
    vn_mean <- n_beta_1 + n_beta_2*df_simulation_field$covariate
    
  }


  
  df_simulation_field$signal   <-  vn_simulated_spatial + vn_mean
  
  
  if(bo_show_plots)
  {
    plot(df_simulation_field$x, 
         df_simulation_field$y,
         cex = 0.5,
         pch = 16,
         main = "Covariate",
         xlab = "Easting",
         ylab = "Northing",
         asp = 1,
         col = viridis(100)[
           rescale(df_simulation_field$covariate, to=c(1,99)
           )]
    )
  }
  

  if(bo_show_plots)
  {
    plot(df_simulation_field$x, 
         df_simulation_field$y,
         cex = 0.5,
         pch = 16,
         main = "Signal",
         xlab = "Easting",
         ylab = "Northing",
         asp = 1,
         col = viridis(100)[
           rescale(df_simulation_field$signal, to=c(1,99)
           )]
    )
  }
  
  ## Same plot, but to add more details  
  if(bo_show_plots)
  {
    plot(df_simulation_field$x, 
         df_simulation_field$y,
         cex = 0.5,
         pch = 16,
         xlab = "Easting",
         ylab = "Northing",
         asp = 1,
         col = viridis(100)[
           rescale(df_simulation_field$signal, to=c(1,99)
           )]
    )
  }
  
  ### devide into rectangular areas; also create polygones for later use
  
  vn_of_observed_areas <- li_general_settings$li_simulation_generate$vn_of_observed_areas
  
  vn_x_area_borders <- round(
    seq(from = min(df_simulation_field$x), 
        to = max(df_simulation_field$x),
        length.out = vn_of_observed_areas [1] + 1
    )
  )
  
  vn_y_area_borders <- round(
    seq(from = min(df_simulation_field$y), 
        to = max(df_simulation_field$y),
        length.out = vn_of_observed_areas[2] + 1
    )
  )
  
  if(bo_show_plots)
  {
    abline(v = vn_x_area_borders, col = "blue")
    abline(h = vn_y_area_borders, col = "blue")
  }  
  
  vn_area_ids <- 1:prod(vn_of_observed_areas)
  
  
  df_borders <-   expand.grid(vn_x_area_borders, 
                              vn_y_area_borders)
  
  vn_area_id <- vector( mode="integer", length = nrow(df_simulation_field) )
  
  li_polygons <- list()
  
  n_area_id <- 1
  
  for (i in 1:vn_of_observed_areas[1])
  {
    for ( j in 1:vn_of_observed_areas[2])
    {
      n_low_limit_x <- vn_x_area_borders[i]
      n_upp_limit_x <- vn_x_area_borders[i+1]
      
      n_low_limit_y <- vn_y_area_borders[j]
      n_upp_limit_y <- vn_y_area_borders[j+1]
      
      vn_area_id <- 
        ifelse(df_simulation_field$x > n_low_limit_x & 
                 df_simulation_field$x < n_upp_limit_x &
                 df_simulation_field$y > n_low_limit_y & 
                 df_simulation_field$y < n_upp_limit_y,
               n_area_id,
               vn_area_id 
        )
      # note that we, for simplicity, do not condider locations on the area borders
      
      if(bo_show_plots)
      {
        # x = mean( c(n_low_limit_x, n_upp_limit_x) ),
        # y = mean( c(n_low_limit_y, n_upp_limit_y) ),
        
          text( x = n_low_limit_x,
                y = n_low_limit_y,
                adj = c(-0.25,-0.25),
                labels = n_area_id,
                cex = 2,
                col = "orange"
          )           
      }
      
      
      pol <-                      Polygon(rbind(c(n_low_limit_x, n_low_limit_y),
                                                c(n_low_limit_x, n_upp_limit_y),
                                                c(n_upp_limit_x, n_upp_limit_y),
                                                c(n_upp_limit_x, n_low_limit_y)
      )
      )
      
      li_polygons[[n_area_id]] <-     Polygons(list(pol), n_area_id)
      
      n_area_id <- 1 + n_area_id
    }
    
  }
  
  
 
  
  
  #spplot(sppoldf, zcol = "area_id")
  
  
  
  
  
  df_simulation_field$area <- vn_area_id
  
  # plot(df_simulation_field$x, 
  #      df_simulation_field$y,
  #      col = viridis(100)[
  #        rescale(df_simulation_field$area, to=c(1,99)
  #        )]
  # )
  # 
  ## Calculate area means ####
  
  df_area_means <- aggregate(x = df_simulation_field$signal,
                             by = list(df_simulation_field$area),
                             FUN = mean
                             )
  
  df_area_mean_covariate <- aggregate(x = df_simulation_field$covariate,
                                      by = list(df_simulation_field$area),
                                      FUN = mean
                                )
  
  df_area_means$area_covariate_mean <- df_area_mean_covariate$x
  
  df_area_means <- df_area_means[-1,] 
  # remove group zero, or: the mean of the not assigned points
  
  names(df_area_means) <- c("area_id", "area_mean", "area_covariate_mean")
  
  
  #### Create polygone object ####
  
  sppol <- SpatialPolygons(li_polygons
  )
  
  row.names(df_area_means) <- df_area_means$area_id
  sppoldf_area_means <- SpatialPolygonsDataFrame( sppol, 
                                                  data = df_area_means 
  )
  
  
  if(bo_show_plots)
  {
    
    ### Just as quick and dirty check: plot means over area
    n_area_id <- 1
    for (i in 1:vn_of_observed_areas[1])
    {
      for ( j in 1:vn_of_observed_areas[2])
      {
        n_low_limit_x <- vn_x_area_borders[i]
        n_upp_limit_x <- vn_x_area_borders[i+1]
        
        n_low_limit_y <- vn_y_area_borders[j]
        n_upp_limit_y <- vn_y_area_borders[j+1]
        
        # note that we, for simplicity, do not consider locations on the area borders
        
        
        points( x = mean( c(n_low_limit_x, n_upp_limit_x) ),
                y = mean( c(n_low_limit_y, n_upp_limit_y) ),
                cex = 5,
                pch = 22,
                col = "white",
                bg = viridis(100)[
                  rescale(df_area_means$area_mean[n_area_id], 
                          from = range(df_simulation_field$signal),
                          to   = c(1,99)
                  )]
        )     
        
        # cat("\n", rescale(df_area_means$area_mean[n_area_id], 
        #                   from = range(df_simulation_field$signal),
        #                   to   = c(1,99)
        # )
        #)
        
        n_area_id <- 1 + n_area_id
      }
      
    }
  }
  
  return(
    list(
      df_area_means       = df_area_means,
      df_simulation_field = df_simulation_field,
      sppoldf_area_means  = sppoldf_area_means
    )
  )
  
  
}  
  
