run_multiple_simulations <- function(li_general_settings)
  ## Runs several simulations in parallel
{


#li_simulation_assess <- li_general_settings$li_simulation_assess 

n_runs          <- li_general_settings$li_multiple_simulations$n_runs

source("run_single_simulation_several_methods.R")


n_cores <- detectCores()

if (n_cores < 9 & n_cores > 1)
  n_cores <-  n_cores-1  
# To have one core free in case of a normal werkstation


cl <- makeCluster(n_cores)
vs_variables <- c("li_general_settings")
clusterExport(cl, vs_variables)


li_simulation <- parSapply(cl,
                            (1:n_runs),
                            run_single_simulation_several_methods,
                            simplify= FALSE
                           )

stopCluster(cl)

#browser()

#rownames(li_simulation[[1]])

#colnames(li_simulation[[1]])


## prepare the list, each element a df with simulation results for a specific method
li_simulations_results <- list()
for (str_approach in rownames(li_simulation[[1]]) )
{
  li_simulations_results[[str_approach]] <- data.frame(
      matrix(NA, 
             nrow = n_runs, 
             ncol = dim(li_simulation[[1]])[2])
    )
  colnames(li_simulations_results[[str_approach]]) <- colnames(li_simulation[[1]])
}


## fill the list with simulation results
for (i_listitem in 1:n_runs )
{
  for (str_method in rownames(li_simulation[[1]]) )
  {
    li_simulations_results[[str_method]][i_listitem,] <- li_simulation[[i_listitem]][str_method,]
  }

}  


## Summarising statistics:
#  calculate mean and sd


## prepare the summarising dataframe
df_simulation_summary <- data.frame(
  matrix(NA, 
         nrow = dim(li_simulation[[1]])[1], 
         ncol = dim(li_simulation[[1]])[2]
         ) 
  )
colnames(df_simulation_summary) <- c(colnames(li_simulation[[1]]))
rownames(df_simulation_summary) <- rownames(li_simulation[[1]])

df_simulation_means <- df_simulation_summary
df_simulation_sd  <- df_simulation_summary


for (str_approach in rownames(li_simulation[[1]]) )
{
  vn_means <- colMeans(li_simulations_results[[str_approach]])[1:12]
  vn_sds <- sapply(li_simulations_results[[str_approach]], sd)[c(1:8, 12)]
  
  df_simulation_means[str_approach,] <- vn_means
  df_simulation_sd[str_approach,] <- c(vn_sds[1:8], NA, NA, NA, vn_sds[12])
    
  df_simulation_summary[str_approach,c(1:8, 12)] <- 
    paste0(
      signif(vn_means[c(1:8,12)], digits = 2), 
      " (",
      signif(vn_sds, digits = 2),
      ")"
    )
  df_simulation_summary[str_approach,9:11] <-
    signif(
      vn_means[9:11],
      digits = 3
    )
  
}


return (
  list(
    df_simulation_summary = df_simulation_summary,
    df_simulation_means   = df_simulation_means,
    df_simulation_sd      = df_simulation_sd,
    li_simulation         = li_simulation
  )
)

}
