
show_1D_simulation_data_and_prediction_results <- function(li_simulation_data, 
                                                        li_estimations_and_predictions_REML = NULL,
                                                        li_estimations_and_predictions_BAK  = NULL,
                                                        li_export = NULL,
                                                        vn_xlim   = NULL,
                                                        vn_ylim   = NULL,
                                                        str_alternative_file_name = NULL,
                                                        bo_use_as_insert  = FALSE,
                                                        bo_bw   = FALSE  # black and white or colour
                                                       ) 
  ## Shows the simulation data, the area means, and the prediction results;
  #  eventual export plots to pdf files  
{  
  
  .pardefault <- par(no.readonly = T) # To be able to reset graphical parameters
    
  if (!missing(li_export))
  {
    bo_plots_to_latex <- li_export$bo_plots_to_latex
    if (is.null(str_alternative_file_name))
    {
      str_file_name     <- "simulation-and-predictions.pdf"
    } else
    {
      str_file_name     <- str_alternative_file_name
    }
    
  } else
  {
    bo_plots_to_latex <- FALSE
  }
  
  if(bo_plots_to_latex)
  {
    pdf( file = paste0(li_export$latex_plot_dir, str_file_name),
         width = 7,
         height = 5)
  }
  
  bo_show_data_only <- TRUE
  bo_show_REML      <- FALSE   
  bo_show_BAK       <- FALSE    
  
  if (!missing(li_estimations_and_predictions_REML)){
    bo_show_data_only <- FALSE
    bo_show_REML      <- TRUE       
  }
  
  if (!missing(li_estimations_and_predictions_BAK)){
    bo_show_data_only <- FALSE
    bo_show_BAK       <- TRUE       
  }  
  
  #browser()
  
  df_data_points <- li_simulation_data$df_data_points
  vn_area_ids     <- li_simulation_data$vn_area_ids 
  df_data_area   <- li_simulation_data$df_data_area
  
  
  #### set margins, font, colors etc. ####
  
  #windowsFonts(Verdana = windowsFont("Verdana"))
  #gpar(fontfamily = "Verdana")
  #par(family = "Verdana")
  
  #https://www.wur.nl/upload_mm/c/e/4/0ecdc66e-f24d-40bc-bbeb-ee2c3561433f_4_WageningenUR_Colour_Reference_Card_2.0_sept2013.pdf
  # define wur colors
  
  if(bo_bw)
  {
    col_sim      <- "black"
    col_obs      <- "black"
    col_reml     <- "black"
    col_pab      <- "black"
  } else
  {
    col_sim      <- "black"
    col_obs      <- rgb(256,   0,   0, 256, maxColorValue = 256) 
    col_reml     <- rgb( 52, 178,  51, maxColorValue = 256) 
    col_pab      <- rgb(  0,  84, 159, maxColorValue = 256)
  }
  
  par(mar=c(4.5, 3.2, 1, 3.2),
      lend = "square")
  par(mfrow=c(1,1))
  
  #### Basic plot ####
  plot(x   = df_data_points$s1,
       y   = df_data_points$z, 
       pch = 20,
       xlab = "",
       ylab = "",
       #yaxt = "n",
       xlim = vn_xlim,
       ylim = vn_ylim,
       las = 1,
       cex = 0.25,
       col = col_sim, 
       las = 1,
       main = "") #Area to point: One dimensional simulation
  grid()
  
  if (!bo_use_as_insert)
  {
    mtext(text = expression("Distance [au]"),
          side = 1,
          line = 2.2,
          cex  = 1.5)
    # expression(bar(z) ~ ~ z[0] ~~ v[0]),
    
    
    mtext(text = "      Variable of interest", 
          side = 2,
          line = 2.1,
          cex  = 1.5,
          las  = 0)
  }
  
  # axis(side = 2,
  #      at = c(0, 5, 10, 15),
  #      labels = c(0, 5, 10, 15),
  #      las = 1
  # )
  # 
  #plot(data_domain$mz)
  
  
  #### BAK predictions ####
  
  if (bo_show_BAK)
  {
    vn_pred_pab <- li_estimations_and_predictions_BAK$z0
    vn_sd_pab   <- sqrt(li_estimations_and_predictions_BAK$v0)
    
    lines(x   = df_data_points$s1,
          y   = vn_pred_pab, 
          col = col_pab  
    )
    
    lines(x   = df_data_points$s1,
          y   = vn_pred_pab - vn_sd_pab, 
          col = col_pab  
    )    
    
    lines(x   = df_data_points$s1,
          y   = vn_pred_pab + vn_sd_pab, 
          col = col_pab  
    )
    
    
  }    
  
  #### REML predictions ####
  
  if (bo_show_REML)
  {
    vn_pred_reml <- li_estimations_and_predictions_REML$vn_pred #li_estimations_and_predictions_REML$vn_zpred_reml
    vn_sd_reml   <- li_estimations_and_predictions_REML$vn_sd    #li_estimations_and_predictions_REML$vn_sd_reml
    #browser()
    lines(x   = df_data_points$s1,
          y   = vn_pred_reml, 
          lty = "dashed",
          lwd = 1.5,
          col = col_reml  
    )
    
    lines(x   = df_data_points$s1,
          y   = vn_pred_reml - vn_sd_reml, 
          lty = "dashed",
          col = col_reml  
    )    
    
    lines(x   = df_data_points$s1,
          y   = vn_pred_reml + vn_sd_reml, 
          lty = "dashed",
          col = col_reml  
    )
    
    
  }  
  
  
  
  #### Area means ####
  
  for (i in vn_area_ids )
  {
    vn_range <- range(df_data_points$s1[df_data_points$area ==  i])
    lines( x = vn_range, 
           y = rep(df_data_area$mz[i],2), 
           col = col_obs, 
           lwd = 2)
    
  }
  
  
  if (!bo_use_as_insert)
  {    
    #### Legend ####  
    vs_legend <- c("Simulated original signal",
                   "Area means / observations")  
    vc_coldefs <- c(col_sim,
                    col_obs)
    vs_pch     <- c(20, NA)
    vs_lty     <- c(NA, 
                    "solid")
    vn_lwd     <- c(2, 2)
    
    if (bo_show_REML){
      vs_legend <- c(vs_legend, 
                     expression(
                       paste("Prediction REML ", phantom(.)%+-%phantom(.), "1 x pred. sd" )
                     )
      )
      vc_coldefs <- c(vc_coldefs,
                      col_reml)
      vs_pch     <- c(vs_pch, 
                      NA)
      vs_lty     <- c(vs_lty,
                      "dashed")
      vn_lwd     <- c(vn_lwd, 1)
    }
    
    if (bo_show_BAK){
      vs_legend <- c(vs_legend, 
                     expression(
                       paste("Prediction Bayesian ", phantom(.)%+-%phantom(.), "1 x pred. sd" )
                     )
      )
      vc_coldefs <- c(vc_coldefs,
                      col_pab)
      vs_pch     <- c(vs_pch, 
                      NA)
      vs_lty     <- c(vs_lty,
                      "solid")
      vn_lwd     <- c(vn_lwd, 1)
    }
    
    
    
    
    legend(x="bottomright",
           legend = vs_legend,
           col = vc_coldefs,
           pch = vs_pch,
           lty = vs_lty,
           lwd = vn_lwd,
           bg  = "white"
    ) 
    
  }
  if(bo_plots_to_latex)
  {
    dev.off()      
  }
  
  par(.pardefault) # reset graphical parameters
    
}


