show_case_data <- function(li_yields_covars_predmask,
                           spdf_prediction   = NULL,
                           li_export       = NULL, 
                           bo_show_hist    = FALSE,  # histogram of yields
                           bo_show_yields  = TRUE,   # map of yields
                           bo_show_covars  = FALSE,  # combined maps of selected covariates, in case of sorghum
                           str_casestudy   = NULL,   # sorghum, millet, PAR 
                           str_title_add   = "",     #  for the prediction
                           vn_xlim         = NULL,   # in case we want a detail area
                           vn_ylim         = NULL,
                           bo_return_li_map_additions = FALSE,
                           bo_is_detail = FALSE  # in case we want a detail area
                           )
## Shows the orgiginal data, eventual predictions, prediction error uncertainties
# as maps and eventually as histogram; eventual exports plots to pdf files

{
    
  
  ## for testing
  # li_yields_covars_predmask = li_yields_covars_predmask
  # spdf_prediction   = spdf_prediction_REML 
  # li_export       = NULL 
  # bo_show_hist    = TRUE  # histogram of yields
  # bo_show_yields  = TRUE   # map of yields/voi
  # bo_show_covars  = TRUE  # combined maps of selected covariates, in case of sorghum
  # str_casestudy   = "PAR"    # sorghum, millet, PAR 
  # str_title_add   = "REML" 
  
  spdf_pred_grid_with_cov  <- li_yields_covars_predmask$spdf_pred_grid_with_cov
  df_yields                <- li_yields_covars_predmask$df_yields
  #spoldf_areas_yld         <- li_yields_covars_predmask$spoldf_areas_yld
  spoldf_areas             <- li_yields_covars_predmask$spoldf_areas
  vn_zbar                  <- li_yields_covars_predmask$vn_zbar  
  
  str_proj_constant_coordinate_intervals <- 
    "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"  
  
  spoldf_areas     <- li_yields_covars_predmask$spoldf_areas
  
  spoldf_areas <- spTransform(spoldf_areas, 
                              CRSobj = str_proj_constant_coordinate_intervals) 
  

  
  spdf_pred_grid_with_cov <- spTransform(spdf_pred_grid_with_cov, 
                              CRSobj = str_proj_constant_coordinate_intervals) 
  
  suppressWarnings(  # Otherwise NA's produce a warning
  spixdf_pred_grid_with_cov <- SpatialPixelsDataFrame(
                                    spdf_pred_grid_with_cov, 
                                    data = spdf_pred_grid_with_cov@data,
                                    tolerance = 1E-2
                                    )
  )
  
  
  
  
  str_zcol_name <- str_casestudy
  
  if (missing(spdf_prediction) | is.null(spdf_prediction))
  {
    bo_plot_prediction <- FALSE
  } else
  {  
    bo_plot_prediction <- TRUE
    spdf_prediction <- spTransform(spdf_prediction, 
                                   CRSobj = str_proj_constant_coordinate_intervals) 
    
    suppressWarnings(  # Otherwise NA's produce a warning
      spixdf_prediction <- SpatialPixelsDataFrame(
        spdf_prediction, 
        data = spdf_prediction@data,
        tolerance = 1E-2
      )
    )
    
  }
  
  
  if (str_casestudy == "sorghum")
  {
    ## Note that some plot limits values are automatically derived, 
    #  some are hard coded for the specific case
    n_voi_min <- 0.35  
    n_voi_max <- 2.2 
    if(bo_plot_prediction)
    {
      n_min_sd <- min(spixdf_prediction$prediction_sd)
      n_max_sd <- quantile(spixdf_prediction$prediction_sd, probs = 0.99) # to get rid of single extreme points
    }  
    # For maps of both yields and predictions
    exp_units <- expression(
      paste("[t ",ha^-1,"]")
    )    
    
    
    
  } else if (str_casestudy == "millet")
  {

    if (bo_is_detail)
    {
      n_min_sd <- 0.05 #min(spixdf_prediction$prediction_sd)
      n_max_sd <- 0.25 #quantile(spixdf_prediction$prediction_sd, probs = 0.99) # to get rid of single extreme points
      n_voi_min <- 0.4# 0.17  
      n_voi_max <- 2 
    } else
    {
      n_min_sd <- 0.05 #min(spixdf_prediction$prediction_sd)
      n_max_sd <- 0.25 #quantile(spixdf_prediction$prediction_sd, probs = 0.99) # to get rid of single extreme points
      n_voi_min   <- 0.4 # 0.17  
      n_voi_max <- 2       
      
    }
    
    exp_units <- expression(
      paste("[t ",ha^-1,"]")
    )    
    

  } else if (str_casestudy == "PAR")
  {
    n_voi_min   <- 25 
    n_voi_max <- 80 
    if(bo_plot_prediction)
    {    
      n_min_sd <- min(spixdf_prediction$prediction_sd)
      n_max_sd <- quantile(spixdf_prediction$prediction_sd, probs = 0.99) # to get rid of single extreme points
    }  
    str_zcol_name <- "voi"
    exp_units <- "[-]"
    
    
  }
  

  
  if (missing(li_export) | is.null(li_export))
  {
    bo_plots_to_latex <- FALSE
    
  } else
  {  
    li_file_names     <- list()
    bo_plots_to_latex <- li_export$bo_plots_to_latex

    li_file_names[["histogram-area-voi"]]   <- "histogram-area-voi.pdf"
    li_file_names[["covariates"]]           <- "covariates.pdf"
    
    li_file_names[["area-voi"]]             <- "area-voi.pdf"
    li_file_names[["prediction-voi"]]       <- "prediction-voi.pdf"
    li_file_names[["prediction-sd-voi"]]    <- "prediction-sd-voi.pdf"        
    
    li_file_names[["area-voi"]]             <- "a2p-yields-millet.pdf"
    li_file_names[["prediction-voi"]]       <- "a2p-prediction-map-millet.pdf"
    li_file_names[["prediction-sd-voi"]]    <- "a2p-prediction-sd-map-millet.pdf"        

    
    if(bo_is_detail)
    {
      li_file_names[["area-voi"]]             <- "a2p-yields-millet-detail.pdf"
      li_file_names[["prediction-voi"]]       <- "a2p-prediction-map-millet-detail.pdf"
      li_file_names[["prediction-sd-voi"]]    <- "a2p-prediction-sd-map-millet-detail.pdf"        
    }
    
        
  }  
  

  if(is.null(vn_xlim) )
  {
      vn_xlim      <-  c(-5.6,2.5)
      vn_ylim      <-  bbox(spixdf_pred_grid_with_cov)[2,]
  }    

#browser()
#### define map additions ####

if(bo_is_detail)
{
  li_arrow = list("SpatialPolygonsRescale", layout.north.arrow(), 
                  offset = c(-2.5,10.5), scale = 0.3)
  li_scale = list("SpatialPolygonsRescale", layout.scale.bar(), 
                  offset = c(-2.7,10.3), scale = 0.905/4, fill=c("transparent","black"))
  li_text_scale_1 = list("sp.text", c(-2.7,10.23), "0", cex=0.7)
  li_text_scale_2 = list("sp.text", c(-2.4,10.23), "25 km", cex=0.7)
} else
{
  li_arrow = list("SpatialPolygonsRescale", layout.north.arrow(), 
                  offset = c(-4.7,14.2), scale = 0.7)
  li_scale = list("SpatialPolygonsRescale", layout.scale.bar(), 
                  offset = c(-5,14), scale = 0.905, fill=c("transparent","black"))
  li_text_scale_1 = list("sp.text", c(-5,13.8), "0")
  li_text_scale_2 = list("sp.text", c(-4,13.8), "100 km")
}

li_areas <- list("sp.polygons",spoldf_areas,col="darkgrey", lwd=1.5, first = FALSE) 

li_map_additions <- list(
  li_arrow ,
  li_text_scale_1,
  li_text_scale_2,
  li_scale,
  li_areas
)


#li_map_additions <<- li_map_additions

remove(li_arrow,li_scale, li_text_scale_1, li_text_scale_2, li_areas)




#### Histogram ####

if (bo_show_hist)
{
  
  if (str_casestudy == "sorghum")
  {
    if(bo_plots_to_latex)
      str_file_name <- paste0(li_export$latex_plot_dir, li_file_names[["histogram-area-voi"]])     
    str_main      <- "Histogram sorghum yield Burkina Faso"
    vn_yield      <- df_yields$sorghum
    exp_xlab      <- expression(
                          paste("Sorghum yield, area mean [t ",ha^-1,"]")
                      )
  } else if (str_casestudy == "millet")    
  {
    if(bo_plots_to_latex)
     str_file_name <- paste0(li_export$latex_plot_dir, li_file_names[["histogram-area-voi"]])     
    str_main      <- "Histogram millet yield Burkina Faso"    
    vn_yield      <- df_yields$millet
    exp_xlab      <- expression(
                        paste("Millet yield, area mean [t ",ha^-1,"]")
                      )    
  } else if (str_casestudy == "PAR")
  {
    if(bo_plots_to_latex)
      str_file_name <- paste0(li_export$latex_plot_dir, li_file_names[["histogram-area-voi"]])     
      str_main      <- "Histogram PAR"    
      vn_yield      <- vn_zbar
      exp_xlab      <- expression(
      paste("Millet yield, area mean [t ",ha^-1,"]")
    )    
    
  }
 
  
  if(bo_plots_to_latex)
  {
    pdf( file = str_file_name,
         width = 6,
         height = 5)
    str_main <-  ""  # export to latex = for journal = no title
    
  } 
  
  hist(vn_yield,
       main = str_main,
       xlab = exp_xlab
       )    

  if(bo_plots_to_latex)
  {
    dev.off()
  }
}




#str(spoldf_areas_yld)


#### Areas with average yields ####

if (bo_show_yields)
{  


# if (bo_show_sorghum)
# {
#   if(bo_plots_to_latex)
#     str_file_name <- paste0(li_export$latex_plot_dir, li_file_names[["area-voi"]])     
#   str_main      <- "Mean yield per area sorghum"
#   # exp_xlab      <- expression(
#   #   paste("Sorghum yield, area mean [t ",ha^-1,"]")
#   #)
# } else    
# {
#   if(bo_plots_to_latex)
#     str_file_name <- paste0(li_export$latex_plot_dir, li_file_names[["area-voi"]])     
#   str_main      <- "Mean yield per area millet"    
#   # exp_xlab      <- expression(
#   #   paste("Millet yield, area mean [t ",ha^-1,"]")
#   #)    
# } 

  
    if(bo_plots_to_latex)
    {      str_file_name <- paste0(li_export$latex_plot_dir, li_file_names[["area-voi"]])
      str_main      <- ""
    } else
    {
      str_main      <- paste0("Mean yield per area ", str_casestudy)
    }
  

if(bo_plots_to_latex)
{
  pdf( file = str_file_name,
       width = 6,
       height = 5)
  #png(file=paste0(li_export$latex_plot_dir, "tbr.png") )
  str_main <-  ""  # export to latex = for journal = no title
  if(bo_is_detail)
  {
    n_legend_label_x <-  0.97
    n_legend_label_y <-  0.23  
  } else
  {
    n_legend_label_x <-  0.96
    n_legend_label_y <-  0.195  
  }
} else
{
  n_legend_label_x <-  0.95  # lattice might use other coordinates in another graphical device
  n_legend_label_y <-  0.15
  
}


  
  print(

      spplot(spixdf_pred_grid_with_cov, 
             zcol      = str_zcol_name, 
             sp.layout = li_map_additions,
             at        = seq(from = n_voi_min,    # forcing a nice, semi-continious scale
                             to   = n_voi_max, 
                             length.out = 100),
             xlim      = vn_xlim,
             ylim      = vn_ylim,
             scales    = list(draw = TRUE),       
             main      = str_main       
      )
      
  )

  
grid.text(x = n_legend_label_x,  
          y = n_legend_label_y, 
          label = exp_units, 
          just = c("right", "top")
) 

  
# panel.text(x = n_legend_label_x,  
#            y = n_legend_label_y, 
#            label=expression(
#             paste("[t ",ha^-1,"]")
#             )    , 
#            adj=c(0,0)
#            ) 


if(bo_plots_to_latex)
{
  dev.off()
}


} #END bo_show_yields



#### Covariates ####


if(bo_show_covars  )
{
  
    
    if(bo_plots_to_latex)
    {
      
      pdf( file   = paste0(li_export$latex_plot_dir, li_file_names[["covariates"]]) ,
           width  = 12,
           height = 8)
      n_legend_label_x_1 <- 0.45
      n_legend_label_x_2 <- 0.95
      n_legend_label_y_1 <- 0.55
      n_legend_label_y_2 <- 0.05    
      n_legend_vert_dist <- 0.02
    } else
    {
      if(.Platform$OS.type == "windows")
      {
        windows(width=12, height=8)
      }  
      n_legend_label_x_1 <- 0.45
      n_legend_label_x_2 <- 0.95
      n_legend_label_y_1 <- 0.55
      n_legend_label_y_2 <- 0.05    
      n_legend_vert_dist <- 0.02
      
    }  
    
  li_par_settings <-  list(
    layout.heights = list(top.padding    = -2,
                          bottom.padding = -2),
    par.main.text  = list(cex=0.5)
  )
    
  plot(
    suppressWarnings(  # Otherwise NA's produce a warning
      spplot(as(spdf_pred_grid_with_cov , "SpatialPixelsDataFrame"), 
             zcol="CFAPAR_27", #at=seq(0.3,2,length.out = 50) ,
             sp.layout= li_map_additions,
             xlim    = vn_xlim,
             ylim    = vn_ylim,             
             scales=list(draw = TRUE),       
             main = "",
             at = seq(from = 20, to = 80, length.out = 100),
             par.settings = li_par_settings
  
      )
    )  ,
    split = c(1,1,2,2) ,
    more  = TRUE
  )
  
  grid.text(x = n_legend_label_x_1,  
            y = n_legend_label_y_1, 
            label="CFAPAR_27",
            gp = gpar(cex = 0.7),
            just = c("left", "top")
  ) 
  grid.text(x = n_legend_label_x_1,  
            y = n_legend_label_y_1 - n_legend_vert_dist, 
            label="[-]",
            gp = gpar(cex = 0.7),
            just = c("left", "top")
  ) 
  
  
  plot(    
    suppressWarnings(  # Otherwise NA's produce a warning
      spplot(as(spdf_pred_grid_with_cov, "SpatialPixelsDataFrame"), 
             zcol      = "CRAIN_EC_27", #at=seq(0.3,2,length.out = 50) ,
             sp.layout = li_map_additions,
             xlim      = vn_xlim,
             ylim      = vn_ylim,             
             scales    = list(draw = TRUE),       
             main      = "",
             at        = seq(from = 100, to = 700, length.out = 100),
             par.settings = li_par_settings       
      )       
    ),
      
  split = c(1,2,2,2) ,  # second row, first column
  more  = TRUE
  )
  
  
  grid.text(x = n_legend_label_x_1,  
            y = n_legend_label_y_2, 
            label="CRAIN_EC_27",
            gp = gpar(cex = 0.7),
            just = c("left", "top")
  ) 
  grid.text(x = n_legend_label_x_1,  
            y = n_legend_label_y_2 - n_legend_vert_dist, 
            label=expression(
              paste("[mm ",d^-1,"]")
            )   ,
            gp = gpar(cex = 0.7),
            just = c("left", "top")
  ) 
  
  
  
  plot(  
    suppressWarnings(  # Otherwise NA's produce a warning
      spplot(as(spdf_pred_grid_with_cov, "SpatialPixelsDataFrame"), 
         zcol       = "TMIN_EC_21", #at=seq(0.3,2,length.out = 50) ,
         sp.layout  = li_map_additions,
         at         = seq(from = 200, to = 230, length.out = 100),
         xlim       = vn_xlim,
         ylim       = vn_ylim,
         scales     = list(draw = TRUE),       
         main       = "",
         par.settings = li_par_settings       
      )   
  ),
  split = c(2,1,2,2) , # first row, second column
  more  = TRUE
  )
  
  
  grid.text(x = n_legend_label_x_2,  
            y = n_legend_label_y_1, 
            label="TMIN_EC_21",
            gp = gpar(cex = 0.7),
            just = c("left", "top")
  ) 
  grid.text(x = n_legend_label_x_2,  
            y = n_legend_label_y_1 - n_legend_vert_dist, 
            label=expression(
              paste("[",degree,"C]")
            )   ,
            gp = gpar(cex = 0.7),
            just = c("left", "top")
  ) 
  
  spdf_pred_grid_with_cov$PHAQ <- spdf_pred_grid_with_cov$PHAQ10/10
  plot(
    suppressWarnings(  # Otherwise NA's produce a warning
  spplot(as(spdf_pred_grid_with_cov, "SpatialPixelsDataFrame"), 
         zcol      = "PHAQ", #at=seq(0.3,2,length.out = 50) ,
         sp.layout = li_map_additions,
         xlim      = vn_xlim,
         ylim      = vn_ylim,
         scales    = list(draw = TRUE),       
         main      = "",
         at        = seq(from = 5.5, to = 7.5, length.out = 100),
         par.settings = li_par_settings       
     )     
  ),
  split = c(2,2,2,2) , # second row, second column
  more  = FALSE
  )
  
  grid.text(x = n_legend_label_x_2,  
            y = n_legend_label_y_2, 
            label="PHAQ",
            gp = gpar(cex = 0.7),
            just = c("left", "top")
  ) 
  grid.text(x = n_legend_label_x_2,  
            y = n_legend_label_y_2 - n_legend_vert_dist, 
            label = "[pH]",
             # paste("[ph]",*%**%, "10")
             # ),
            gp = gpar(cex = 0.7),
            just = c("left", "top")
  ) 
  
  
  
  if(bo_plots_to_latex)
  {
    dev.off()
  } else
  {
    #dev.off()
  }
}


#### Plot prediction and prediction uncertainty ####

if (bo_plot_prediction)
{  
  
  
  if(bo_plots_to_latex)
  {
    
    pdf( file   = paste0(li_export$latex_plot_dir, li_file_names[["prediction-voi"]]) ,
         width  = 6,
         height = 5)
    str_main      <- ""   
    # n_legend_label_x <-  0.95
    # n_legend_label_y <-  0.15  
  } else
  {
    str_main      <- paste0("Prediction ", str_casestudy, " ", str_title_add)
    n_legend_label_x <-  0.95  # lattice might use other coordinates in another graphical device
    n_legend_label_y <-  0.15
  }
    


  print(
   
      spplot(spixdf_prediction, 
             zcol      = "prediction", 
             sp.layout = li_map_additions,
             at        = seq(from = n_voi_min,    # forcing a nice, semi-continious scale
                             to   = n_voi_max, 
                             length.out = 100),
             xlim      = vn_xlim,
             ylim      = vn_ylim,
             scales    = list(draw = TRUE),       
             main      = str_main       
      )
     
  ) 
  
  grid.text(x = n_legend_label_x,  
            y = n_legend_label_y, 
            label = exp_units, 
            just = c("right", "top")
  )

  if(bo_plots_to_latex)
  {
    
    dev.off()
    
    pdf( file   = paste0(li_export$latex_plot_dir, li_file_names[["prediction-sd-voi"]]) ,
         width  = 6,
         height = 5)
    
    str_main      <- "" 
    # n_legend_label_x <-  0.95
    # n_legend_label_y <-  0.15  
    
  }
  else
  {
    str_main    <-  paste0("Prediction sd ", str_casestudy, " ", str_title_add) 
    n_legend_label_x <-  0.95
    n_legend_label_y <-  0.15  
    
  }
  

  print(

      spplot(spixdf_prediction, 
             zcol      = "prediction_sd", 
             sp.layout = li_map_additions,
             at        = seq(from = n_min_sd,    # forcing a nice, semi-continious scale
                             to = n_max_sd, 
                             length.out = 100),
             xlim      = vn_xlim,
             ylim      = vn_ylim,
             scales    = list(draw = TRUE),       
             main      = str_main       
      )
      
  )    
  
  grid.text(x = n_legend_label_x,  
            y = n_legend_label_y, 
            label = exp_units, 
            just = c("right", "top")
  )
  
  if(bo_plots_to_latex)
  {
    
    dev.off()
    
  }  


    
} # end show prediction and prediction uncertainty

  if (bo_return_li_map_additions)
  {
    return (li_map_additions)
  }  

}


