show_parameter_assessments <- function(li_postParams, 
                                       li_estimations_and_predictions_REML = NULL,
                                       li_simulation_generate = NULL,
                                       li_export = NULL,
                                       bo_show_marginals_postParams  = TRUE, # the build in function in postParams
                                       bo_show_marginals  = FALSE, # plots adjusted for simulated case                                        
                                       bo_show_joint_c1_a = TRUE,
                                       bo_show_joint_beta_a = TRUE,
                                       bo_bw    = FALSE
)
  ## Shows the (Bayesian) inferenced parameters, eventual (REML) estimated parameters 
  # and the true values; eventual exports plots to pdf files
  
{
  
  .pardefault <- par(no.readonly = T) # To be able to reset graphical parameters 
  
  if (missing(li_export) | is.null(li_export))
  {
    bo_use_greek      <- FALSE
    bo_plots_to_latex <- FALSE
    
  } else
  {
    bo_use_greek      <- li_export$use_greek
    bo_plots_to_latex <- li_export$bo_plots_to_latex
    li_file_names     <- list()
    li_file_names[["marginals-spatial"]]   <- "simulation-marginals-spatial.pdf"
    li_file_names[["marginals-trend"]]     <- "simulation-marginals-trend.pdf"
    li_file_names[["joint-spatial"]]       <- "simulation-joint-spatial.pdf"
    li_file_names[["joint-trend-spatial"]] <- "simulation-joint-trend-spatial.pdf" 
    li_file_names[["joint-legend"]]        <- "simulation-joint-legend.pdf" 
    li_file_names[["marginal-c1"]]         <- "marginal-c1.pdf" 
    li_file_names[["marginal-a"]]          <- "marginal-a.pdf" 
    li_file_names[["marginal-betas"]]      <- "marginal-betas.pdf"
  }
  
  if(bo_use_greek)
  {
    ex_range_par    <- expression(phi)
    ex_partial_sill <- expression(sigma^2)
  } else
  {
    ex_range_par    <- expression(a)
    ex_partial_sill <- expression(c[1])    
  }
  
  
  if (missing(li_estimations_and_predictions_REML) | is.null(li_estimations_and_predictions_REML))
  {
    bo_show_REML      <- FALSE       
  } else
  {
    bo_show_REML      <- TRUE
  }
  
  
  if (missing(li_simulation_generate) | is.null(li_simulation_generate))
  {
    bo_show_sim_pars  <- FALSE
  } else  
  {
    bo_show_sim_pars  <- TRUE
    
    vn_simulation_betas <- c(li_simulation_generate$n_beta_1,
                             li_simulation_generate$n_beta_2)
    
  }
  
  
  
  if(bo_plots_to_latex)
  {
    
    # par(cex.lab  = 2, 
    #     cex.axis = 2
    #     )
    n_pdf_pointsize <- 20 # font size scaling in pdfs
    n_hor_ticks <- 4  # the _desired_ number of ticks on the axis
    n_ver_ticks <- 4
    str_ylabel_marginal_after_first <- ""
    
  } else
  {
    n_hor_ticks <- 5
    n_ver_ticks <- 5
    n_pdf_pointsize <- 12
    str_ylabel_marginal_after_first <- "Probability density"
  }
  
  
  if (bo_bw)
  {
    col_reml  <- "black"
    lty_reml  <- "dashed"
    col_obs   <- "black"
    lty_obs   <- "dotdash"   
    col_pab   <- "black"
    lty_pab   <- "solid"     
    col_prior <- "black"
    lty_prior <- "dotted"
    
  } else
  {
    col_reml  <- rgb( 52, 178,  51, maxColorValue = 256) 
    lty_reml  <- "dashed"
    col_obs   <- rgb(256,   0,   0, 256, maxColorValue = 256) 
    lty_obs   <- "dotdash"   
    col_pab   <- rgb(  0,  84, 159, maxColorValue = 256)
    lty_pab   <- "solid"     
    col_prior <- "orange"
    lty_prior <- "dotted"
  }
  
  n_rug_lwd <- 2
  
  #### Legend for marginal non-postParams plots - definition ####  
  draw_legend_marg <- function()
  {
    vs_legend <- c("Posterior",
                   "Prior")  
    vc_cols   <- c(col_pab, 
                   col_prior)
    vs_lty     <- c(lty_pab,
                    lty_prior) 
    
    
    if(bo_show_REML)
    {
      vs_legend <- c(vs_legend,
                     "REML")  
      vc_cols   <- c(vc_cols, 
                     col_reml)
      vs_lty <- c(vs_lty,
                  lty_reml)
    }
    
    
    if(bo_show_sim_pars)
    {
      
      vs_legend <- c(vs_legend,
                     "Simulation")  
      vc_cols   <- c(vc_cols, 
                     col_obs)
      vs_lty <- c(vs_lty,
                  lty_obs)      
      
    }
    
    legend(#xpd = TRUE, # to enable printing outside plot
      x = "topright", 
      legend = vs_legend, 
      col    = vc_cols,
      lty    = vs_lty,
      pt.cex = c(1.5),
      lwd    = c(1, 1.5, n_rug_lwd, n_rug_lwd),
      bg     = "white",
      y.intersp = 0.8,
      bty    = 'n'
    )
  }  
  
  
  #### Legend for joint density plots (if not exported) - definition ####  
  
  draw_legend_joint <- function()
  {
    vs_legend <- c("Max",
                   "Min")  
    vc_cols   <- c("black", 
                   "darkgrey")
    vn_pch <- c(15, 0)
    
    
    if(bo_show_REML | bo_show_sim_pars)
    {
      vs_legend <- c(vs_legend,
                     NA)  
      vc_cols   <- c(vc_cols, 
                     "white")
      vn_pch <- c(vn_pch,
                  NA)
    }
    
    
    
    if(bo_show_REML)
    {
      vs_legend <- c(vs_legend,
                     "REML")  
      vc_cols   <- c(vc_cols, 
                     col_reml)
      vn_pch <- c(vn_pch,
                  4)
    }
    
    
    if(bo_show_sim_pars)
    {
      
      vs_legend <- c(vs_legend,
                     "Simulation")  
      vc_cols   <- c(vc_cols, 
                     col_obs)
      vn_pch <- c(vn_pch,
                  3)      
      
    }
    
    legend(#xpd = TRUE, # to enable printing outside plot
      x = "topright", 
      legend = vs_legend, 
      col    = vc_cols,
      pch    = vn_pch,
      pt.cex = c(1.5),
      title  = "Indicative:",
      bg = "white"
    )
  }
  
  
  
  #### Marginals Postparams####  
  if(bo_show_marginals_postParams)
  {
    
    plotPostCovParams(li_postParams$c1Vec , 
                      li_postParams$f0c1 , 
                      li_postParams$fpc1 , 
                      li_postParams$aVec , 
                      li_postParams$f0a , 
                      li_postParams$fpa , 
                      newPlot = FALSE)
    
    plotPostTrendParams(li_postParams$betaMat , 
                        li_postParams$fpbeta,
                        newPlot = FALSE)
    
    
  }
  par(.pardefault) # reset graphical parameters
  
  #### Marginals non-Postparams ####  
  
  if (bo_show_marginals)  
  {
    #### c1, sigmasq ####
    
    if (max(li_postParams$f0c1) > max(li_postParams$fpc1))
    {
      n_max_c1 <- max(li_postParams$fpc1)*2
    } else
    {
      n_max_c1 <- max(li_postParams$fpc1)
    }
    
    
    if(bo_plots_to_latex)
    {
      pdf( file = paste0(li_export$latex_plot_dir, li_file_names[["marginal-c1"]]),
           width = 6,
           height = 5,
           pointsize = n_pdf_pointsize)
      str_main <-  ""  # export to latex = for journal = no title
      
    } else
    {
      str_main <- 
        bquote("Marginal posterior "* .(ex_partial_sill[[1]]))
    }
    
    vn_pretty_x <- pretty(li_postParams$c1Vec, n = n_hor_ticks-1 )
    vn_pretty_y <- pretty(
      c(li_postParams$fpc1,0,n_max_c1), 
      n = n_ver_ticks-1
    ) #n_ver_ticks-1 )
    
    plot( 
      x    = li_postParams$c1Vec,
      y    = li_postParams$fpc1,
      type = "l",
      col  = col_pab,
      #ylim = c(0, n_max_c1 ),
      xlab = ex_partial_sill,
      ylab = "Probability density",
      las  = 1,
      lty  = lty_pab,
      xlim = range(vn_pretty_x),
      ylim = range(vn_pretty_y),
      xaxp = c(range(vn_pretty_x), length(vn_pretty_x)-1 ),
      yaxp = c(range(vn_pretty_y), length(vn_pretty_y)-1 ),
      main = str_main
    )
    
    
    
    grid()
    
    ## add prior
    lines( 
      x = li_postParams$c1Vec,
      y = li_postParams$f0c1,
      col = col_prior,
      lty = lty_prior,
      lwd = 1.5
    )
    
    if(bo_show_sim_pars)
    {
      rug(x        = li_simulation_generate$n_c1, 
          ticksize = 0.2,
          lwd      = n_rug_lwd,
          col      = col_obs,
          lty      = lty_obs)
    }  
    
    
    ## Show REML estimations
    if (bo_show_REML)
    {
      rug(x        = li_estimations_and_predictions_REML$n_c1, 
          ticksize = 0.2,
          lwd      = n_rug_lwd,
          col      = col_reml,
          lty      = lty_reml)
    }  
    
    
    draw_legend_marg()
    
    
    if(bo_plots_to_latex)
    {
      dev.off()  
    }
    
    #### a ####
    
    
    if(bo_plots_to_latex)
    {
      pdf( file = paste0(li_export$latex_plot_dir, li_file_names[["marginal-a"]]),
           width = 6,
           height = 5,
           pointsize = n_pdf_pointsize)
      str_main <-  ""  # export to latex = for journal = no title
      
    } else
    {
      str_main <- 
        bquote("Marginal posterior "* .(ex_range_par[[1]]) )
    }
    
    if (max(li_postParams$f0a) > max(li_postParams$fpa))
    {
      n_max_a <- max(li_postParams$fpa)*2
    } else
    {
      n_max_a <- max(li_postParams$fpa)
    }
    
    vn_pretty_x <- pretty(li_postParams$aVec, n = n_hor_ticks-1 )
    vn_pretty_y <- pretty(
      c(li_postParams$fpa,0,n_max_a), 
      n = n_ver_ticks-1
    )
    
    plot( 
      x = li_postParams$aVec,
      y = li_postParams$fpa,
      type = "l",
      col = col_pab,
      #ylim = c(0, n_max_a ),
      xlab = ex_range_par,
      ylab = str_ylabel_marginal_after_first,
      las = 1,
      xlim = range(vn_pretty_x),
      ylim = range(vn_pretty_y),
      xaxp = c(range(vn_pretty_x), length(vn_pretty_x)-1 ),
      yaxp = c(range(vn_pretty_y), length(vn_pretty_y)-1 ),    
      lty = lty_pab
    )
    
    grid()
    
    lines( 
      x = li_postParams$aVec,
      y = li_postParams$f0a,
      col = col_prior,
      lwd = 1.5,
      lty = lty_prior
    )
    
    if(bo_show_sim_pars)
    {
      rug(x        = li_simulation_generate$n_a, 
          ticksize = 0.2,
          lwd      = n_rug_lwd,
          col      = col_obs,
          lty      = lty_obs)
    }  
    
    
    ## Show REML estimations
    if (bo_show_REML)
    {
      rug(x        = li_estimations_and_predictions_REML$n_a, 
          ticksize = 0.2,
          lwd      = n_rug_lwd,
          col      = col_reml,
          lty      = lty_reml)
    } 
    
    
    if(bo_plots_to_latex)
    {
      dev.off()
    } else # In our situation, one legend is enough when exporting to latex
    {
      draw_legend_marg()
      
    }
    
    # ex_range_par    <- expression(phi)
    
    #### beta ####
    
    
    n_k <- dim(li_postParams$fpbetaa)[1] # number of trend coefficients
    
    if(bo_plots_to_latex)
    {
      pdf( file = paste0(li_export$latex_plot_dir, li_file_names[["marginal-betas"]]),
           width = 9,
           height = 5,
           pointsize = n_pdf_pointsize)
      str_main <-  ""  # export to latex = for journal = no title
      
    } else
    {
      str_main <- 
        #bquote("Marginal posterior " * beta[.(paste(i,""))])  
        bquote("Marginal posterior " * beta) 
    }    
    
    par(mfrow=c(1,n_k))
    
    
    par(mar=c(5,2.5,0,0.5))
    
    for (i in 1:n_k)
    {
      
      ma_fpbetaa <- li_postParams$fpbetaa[i,,] # select first trend coefficient
      
      # define y max
      
      if (bo_show_REML)
      {
        n_ymax <- dnorm (
          x    = li_estimations_and_predictions_REML$vn_betas[i],
          mean = li_estimations_and_predictions_REML$vn_betas[i],
          sd   = li_estimations_and_predictions_REML$vn_se_betas[i]
        )
      } else
      {
        n_ymax <- max(li_postParams$fpbeta[i,])      
      }
      
      
      vn_pretty_x <- pretty(li_postParams$betaMat[i,], n = n_hor_ticks-1 )
      vn_pretty_y <- pretty(
        c(li_postParams$fpbeta[i,],0,n_ymax), 
        n = n_ver_ticks-1
      )
      
      
      plot( 
        x = li_postParams$betaMat[i,],
        y = li_postParams$fpbeta[i,],
        type = "l",
        col = col_pab,
        #ylim = c(0, n_ymax),
        xlab = bquote(beta[.(paste(i,""))]),
        ylab = str_ylabel_marginal_after_first,
        las = 1,
        xlim = range(vn_pretty_x),
        ylim = range(vn_pretty_y),
        xaxp = c(range(vn_pretty_x), length(vn_pretty_x)-1 ),
        yaxp = c(range(vn_pretty_y), length(vn_pretty_y)-1 ),          
        lty = lty_pab
      )
      
      grid()
      
      
      if(bo_show_sim_pars)
      {
        
        rug(x        = vn_simulation_betas[i], 
            ticksize = 0.2,
            lwd      = n_rug_lwd,
            col      = col_obs,
            lty      = lty_obs
        )
        
      }
      
      
      if (bo_show_REML)
      {
        
        curve( 
          expr = dnorm (x,
                        mean = li_estimations_and_predictions_REML$vn_betas[i],
                        sd   = li_estimations_and_predictions_REML$vn_se_betas[i]
          ),
          from = min(li_postParams$betaMat[i,]),
          to   = max(li_postParams$betaMat[i,]),
          add  = TRUE,
          lwd  = n_rug_lwd,
          col  = col_reml,
          lty  = lty_reml
        )
        
        
      }
      
    }  
    
    
    if(bo_plots_to_latex)
    {
      dev.off()
    } else # In our situation, one legend is enough when exporting to latex
    {
      draw_legend_marg()
      
    }  
    
    
    par(mfrow=c(1,1))
  } # END  if(bo_show_marginals)  
  
  par(.pardefault) # reset graphical parameters
  
  #### Joint c1 a  ####  
  
  
  if (bo_show_joint_c1_a)
  {
    
    if(bo_plots_to_latex)
    {
      pdf( file = paste0(li_export$latex_plot_dir, li_file_names[["joint-spatial"]]),
           width = 5,
           height = 5,
           pointsize = 16)
      str_main <-  ""  # export to latex = for journal = no title
      
    } else
    {
      str_main <- 
        bquote("Joint posterior ("* .(ex_partial_sill[[1]]) *", " * .(ex_range_par[[1]]) * ")")
    }
    
    
    
    #par(las=1)
    
    par(mfrow=c(1,1))
    vn_seq_for_colors <- (seq(from=0, to=1, length.out=101))
    
    vs_colors <- rgb( red   = 1-vn_seq_for_colors,
                      green = 1-vn_seq_for_colors,
                      blue  = 1-vn_seq_for_colors,
                      alpha = 1)#vn_seq_for_colors) 
    
    ma_fpc1a <- li_postParams$fpc1a
    
    ma_fpc1a <- ma_fpc1a/sum(ma_fpc1a, na.rm = TRUE)
    
    
    ma_fpc1a_plot <-ma_fpc1a^0.35 # adjust this for a nice plot
    # adjust this for a nice plot; is a increasing, monotonically increasing function
    
    
    vn_pretty_x <- pretty(li_postParams$c1Vec, n = n_hor_ticks-1 )
    vn_pretty_y <- pretty(li_postParams$aVec,  n = n_ver_ticks-1 )
    
    
    image (x = li_postParams$c1Vec,
           y = li_postParams$aVec,
           z = ma_fpc1a_plot,
           xlab = ex_partial_sill[[1]],
           ylab = "",
           #xlim = c(0, max(li_postParams$c1Vec) ) ,
           col = vs_colors,
           xaxs = "r",
           yaxs = "r",
           xlim = range(vn_pretty_x),
           ylim = range(vn_pretty_y),
           xaxp = c(range(vn_pretty_x), length(vn_pretty_x)-1 ),
           yaxp = c(range(vn_pretty_y), length(vn_pretty_y)-1 ),  
           las  = 1,
           main = str_main
    )
    
    mtext(ex_range_par[[1]], 
          side=2, 
          line=3,
          las=2)
    
    grid()
    
    
    
    if(bo_show_sim_pars)
    {
      
      points(x   = li_simulation_generate$n_c1,
             y   = li_simulation_generate$n_a,
             pch = 3,
             lwd = 5,
             col = "white",
             cex = 3
      )
      
      points(x   = li_simulation_generate$n_c1, 
             y   = li_simulation_generate$n_a,
             pch = 3,
             col = col_obs,
             cex = 3
      )        
      
    }    
    
    ## Show REML estimations
    if (bo_show_REML)
    {
      
      points(x   = li_estimations_and_predictions_REML$n_c1,
             y   = li_estimations_and_predictions_REML$n_a,
             pch = 4,
             lwd = 4,
             col = "white",
             cex = 3
      )      
      
      points(x   = li_estimations_and_predictions_REML$n_c1,
             y   = li_estimations_and_predictions_REML$n_a,
             pch = 4,
             col = col_reml,
             cex = 3
      )
      
    }
    
    
    
    if(!bo_plots_to_latex) 
       {
          draw_legend_joint()
       }
    if(bo_plots_to_latex)
    {
      dev.off() 
    }
    
  }  
  
  
  par(.pardefault) # reset graphical parameters
  
  #### Joint beta a  ####  
  if (bo_show_joint_beta_a)
  {
    
    #browser()
    
    if(bo_plots_to_latex)
    {
      pdf( file = paste0(li_export$latex_plot_dir, li_file_names[["joint-trend-spatial"]]),
           width = 7,
           height = 5,
           pointsize = n_pdf_pointsize)
      str_main <-  ""  # export to latex = for journal = no title
      
    } else
    {
      str_main <- ""
      #   bquote("Joint posterior (" * beta[.(paste(i,""))]  * ", "  * .(ex_range_par[[1]]) * ")" )
    }
    
    
    n_k <- dim(li_postParams$fpbetaa)[1] # number of trend coefficients
    
    vn_seq_for_colors <- (seq(from=0, to=1, length.out=101))
    
    vs_colors <- rgb( red   = 1-vn_seq_for_colors,
                      green = 1-vn_seq_for_colors,
                      blue  = 1-vn_seq_for_colors,
                      alpha = 1)#vn_seq_for_colors) 
    
    
    par(mfrow=c(1,n_k))
    par(mar=c(5,3.5,1,1))
    
    
    
    for (i in 1:n_k)
    {
      
      
      ma_fpbetaa <- li_postParams$fpbetaa[i,,] # select first trend coefficient
      
      ma_fpbetaa <- ma_fpbetaa/sum(ma_fpbetaa, na.rm = TRUE)
      
      
      ma_fpbetaa_plot <- ma_fpbetaa^0.35 
      # adjust this for a nice plot; is a increasing, monotonically increasing function
      # 
      
      vn_pretty_x <- pretty(li_postParams$betaMat[i,]*0.8, n = n_hor_ticks-1 )
      vn_pretty_y <- pretty(li_postParams$aVec,  n = n_ver_ticks-1 )
      
      
      image (x = li_postParams$betaMat[i,],
             y = li_postParams$aVec,
             z = ma_fpbetaa_plot,
             #z = ma_fpbetaa[,],
             xlab = bquote(beta[.(paste(i,""))]),
             ylab = "",#expression(phi),
             col = vs_colors,
             xlim = range(vn_pretty_x),
             ylim = range(vn_pretty_y),
             xaxp = c(range(vn_pretty_x), length(vn_pretty_x)-1 ),
             yaxp = c(range(vn_pretty_y), length(vn_pretty_y)-1 ),             
             xaxs = "r",
             yaxs = "r",
             #axes=FALSE, 
             #frame.plot=TRUE,
             las  = 1,
             main = str_main
      )
      
      
      mtext(ex_range_par[[1]], 
            side=2, 
            line=3,
            las=2)
      
      grid()
      
      
      if(bo_show_sim_pars)
      {
        
        points(x   = vn_simulation_betas[i],
               y   = li_simulation_generate$n_a,
               pch = 3,
               lwd = 5,
               col = "white",
               cex = 3
        )
        
        points(x   = vn_simulation_betas[i],
               y   = li_simulation_generate$n_a,
               pch = 3,
               col = col_obs,
               cex = 3
        )        
        
        
        
        
      }
      
      
      
      if (bo_show_REML)
      {
        points(x   = li_estimations_and_predictions_REML$vn_betas[i],
               y   = li_estimations_and_predictions_REML$n_a,
               pch = 4,
               lwd = 4,
               col = "white",
               cex = 3
        )
        
        points(x   = li_estimations_and_predictions_REML$vn_betas[i],
               y   = li_estimations_and_predictions_REML$n_a,
               pch = 4,
               col = col_reml,
               cex = 3
        )        
        
        
      }
      
      if (i == n_k &  !bo_plots_to_latex ) # plot legend only once, and not at all in case of LaTex
      {
        draw_legend_joint()
      }
    }
    
    if(bo_plots_to_latex)
    {
      dev.off() 
    }    
    
    
  } 
  
  
  ## Separate joint legend for export ####
  
  if(bo_plots_to_latex)
  {
    par(mfrow = c(1,1))
    par(mar =  c(0,0,0,0),
        oma =  c(0,0,0,0)
        )
    #par(.pardefault)
    pdf( file = paste0(li_export$latex_plot_dir, li_file_names[["joint-legend"]]),
         width = 4,
         height = 4,
         pointsize = 15)

    vs_legend <- c("Posterior density")  
    vc_cols   <- c(NA)
    vn_pch    <- c(NA)
    
    
    if(bo_show_REML)
    {
      vs_legend <- c(vs_legend,
                     "REML")  
      vc_cols   <- c(vc_cols, 
                     col_reml)
      vn_pch <- c(vn_pch,
                  4)
    }
    
    
    if(bo_show_sim_pars)
    {
      
      vs_legend <- c(vs_legend,
                     "Simulation")  
      vc_cols   <- c(vc_cols, 
                     col_obs)
      vn_pch <- c(vn_pch,
                  3)      
      
    }
    
    #plot.new()
    plot(NULL ,
         xaxt='n',
         yaxt='n',
         bty='n',
         ylab='',
         xlab='', 
         xlim=0:1, 
         ylim=0:1)
    
    legend(#xpd = TRUE, # to enable printing outside plot
      x = "bottomleft", 
      legend = vs_legend, 
      col    = vc_cols,
      pch    = vn_pch,
      pt.cex = 1.5,
      bg = "white"
    )
    
    gradient.rect(xleft   = -.02,
                  ybottom = 0.37,
                  xright  = 0.12,
                  ytop    = 0.50,
                  # reds,
                  # greens,
                  # blues,
                  col= smoothColors("black",200,"white"), #c("black","grey"),
                  #nslices=50,
                  gradient="x",
                  border = "white"
    )
  

    dev.off() 
  }    
  
  #system("open ./exportcolour/simulation-joint-legend.pdf")
  
  
  par(.pardefault) # reset graphical parameters
  
  
  
}
