## Plots one or two reference priors

show_reference_prior <- function(
  li_posteriors_and_predictions_BAK,
  str_context = "none",  # simulations or case
  li_posteriors_and_predictions_BAK_2 = NULL,
  li_export = NULL,
  vn_xlim   = NULL,
  vn_ylim   = NULL,
  str_alternative_file_name = NULL
) 
{  
  
  ## str_context = "simulations", "case" or none
  # only influences the (predefined) legends and the x axis label
  ## For testing
  # li_export <- li_general_settings$li_export
  
  
  if (!missing(li_export))
  {
    bo_use_greek      <- li_export$use_greek
    bo_plots_to_latex <- li_export$bo_plots_to_latex
    if (is.null(str_alternative_file_name))
    {
      str_file_name     <- "reference-prior.pdf"
    } else
    {
      str_file_name     <- str_alternative_file_name
    }
    
  } else
  {
    bo_use_greek      <- FALSE
    bo_plots_to_latex <- FALSE
  }
  
  if(bo_plots_to_latex)
  {
    pdf( file = paste0(li_export$latex_plot_dir, str_file_name),
         width = 7,
         height = 5)
  }
  
  if (str_context == "simulations")
  {
    
    if(bo_use_greek)
    {
      ex_range_par    <- expression(phi~"[au]")
    } else
    {
      ex_range_par    <- expression(a~"[au]")
    }
    
    
  } else if  (str_context == "case")
  {
    
    if(bo_use_greek)
    {
      ex_range_par    <- expression(phi~"[m]")
    } else
    {
      ex_range_par    <- expression(a~"[m]")
    }
    
  } else
  {
    
    if(bo_use_greek)
    {
      ex_range_par    <- expression(phi)
    } else
    {
      ex_range_par    <- expression(a)
    }    
    
  }
  
  
  if(missing(li_posteriors_and_predictions_BAK_2))
  {
    bo_plot_2nd <- FALSE  
    
    
    
    
  }  else
  {
    bo_plot_2nd <- TRUE   
    
  }
  
  
  
  vn_range          <- li_posteriors_and_predictions_BAK$aVec
  vn_range_density  <- li_posteriors_and_predictions_BAK$f0a 
  if(bo_plot_2nd)
  {
    vn_range_density_2  <- li_posteriors_and_predictions_BAK_2$f0a 
  }
  
  
  #### set margins, font, colors etc. ####
  
  #windowsFonts(Verdana = windowsFont("Verdana"))
  #gpar(fontfamily = "Verdana")
  #par(family = "Verdana")
  
  #https://www.wur.nl/upload_mm/c/e/4/0ecdc66e-f24d-40bc-bbeb-ee2c3561433f_4_WageningenUR_Colour_Reference_Card_2.0_sept2013.pdf
  # define wur colors
  
  col_prior <- "orange"
  lty_prior <- "dotdash"
  
  par(mar=c(4.5, 4.2, 1, 3.2),
      lend = "square")
  par(mfrow=c(1,1))
  
  #### Basic plot ####
  plot(x   = vn_range,
       y   = vn_range_density, 
       type = 'l',
       xlab = ex_range_par,
       ylab = "",
       #yaxt = "n",
       #xlim = vn_xlim,
       #ylim = vn_ylim,
       las = 1,
       cex = 0.05,
       col = col_prior, 
       las = 1,
       main = "") #Area to point: One dimensional simulation
  
  
  if(bo_plot_2nd)
  {
    lines(x   = vn_range,
          y   = vn_range_density_2, 
          col = "purple",
          lty = "dotted"
    ) 
  } 
  grid()
  
  #### Legend ####
  
  if (str_context == "case")
  {  
    legend(#xpd = TRUE, # to enable printing outside plot
      x = "topright", 
      legend = c("sorghum (4 covariates)", "millet (no covariates)"),
      col    = c(col_prior, "purple"),
      lty    = c("solid","dashed"),
      bg = "white"
    )
    
  } else if (str_context == "simulations")
  {
    grid()
    legend(#xpd = TRUE, # to enable printing outside plot
      x = "topright", 
      legend = c("simulation 30 areas", "simulation 10 areas" ),
      col    = c(col_prior, "purple"),
      lty    = c("solid","dashed"),
      bg = "white"
    )
  } else
  {
    # no legend
  }
  
  
  
  
  
  
  
  
  
  
  if(bo_plots_to_latex)
  {
    dev.off()      
  }
  
  
}


