show_validation_stats_1D_sim <- function(li_val_stats_BAK,
                                  li_val_stats_REML = NULL,
                                  li_val_stats_baseline = NULL,
                                  bo_show_plots  = TRUE,
                                  li_export      = NULL
)
  ## Returns summary validation statistics, eventually exports a latex table
  # (Table 2 in paper). Plots the prediction residuals, the prediction residuals density,
  # the QQ-plot of the residuals and standardised squared prediction 
  # error density; not used in paper. 
{
  
  
  if (missing(li_val_stats_REML) || is.null(li_val_stats_REML) )
  {
    bo_show_REML      <- FALSE       
  } else
  {
    bo_show_REML      <- TRUE
  }
  
  if (missing(li_val_stats_baseline) || is.null(li_val_stats_baseline) )
  {
    bo_show_basel      <- FALSE       
  } else
  {
    bo_show_basel      <- TRUE
  }

  if (!missing(li_export) || !is.null(li_export))
  {
    bo_plots_to_latex   <- li_export$bo_plots_to_latex
    bo_tables_to_latex  <- li_export$bo_tables_to_latex
    str_table_file_name <- "validation-single-simulation.tex"
    str_path_export_table <- li_export$str_tables_to_latex
  } else
  {
    bo_plots_to_latex  <- FALSE
    bo_tables_to_latex <- FALSE    
  }  
  
  
  if (bo_show_plots)
  {

    
    col_BAK   <- rgb(  0,  84, 159, maxColorValue = 256)
    lty_BAK   <- "solid"  
    
    col_reml  <- rgb( 52, 178,  51, maxColorValue = 256) 
    lty_reml  <- "dashed"
    
    col_basel <- rgb(256,   0,   0, 256, maxColorValue = 256) 
    lty_basel <- "dotted"   
   

    #### Legend definition ####  
    
    bo_show_REML <- TRUE
    
    draw_legend <- function(bo_show_REML  = FALSE,
                            bo_show_basel = FALSE, 
                            str_location  = "topleft",
                            bo_show_chisq = FALSE,
                            bo_show_gauss = FALSE
                           )
    {
      vs_legend <- c("BAK")  
      vc_cols   <- c(col_BAK)
      #vs_lty    <- c("lty_BAK")
    
      
      if(bo_show_REML)
      {
        vs_legend <- c(vs_legend,
                       "REML")
    
        vc_cols   <- c(vc_cols, 
                       col_reml)
        #vs_lty    <- c(vs_lty,
        #               "lty_reml")        
      }
      
      if(bo_show_basel)
      {
        vs_legend <- c(vs_legend,
                       "Baseline")
        
        vc_cols   <- c(vc_cols, 
                       col_basel)
        #vs_lty    <- c(vs_lty,
        #               "lty_reml")        
      }
      
      
      
      if(bo_show_chisq)
      {
        vs_legend <- c(vs_legend,
                       expression(chi[df==1]^2 ))
        
        vc_cols   <- c(vc_cols, 
                       "black")
        
      }
      
      
      if(bo_show_gauss)
      {
        vs_legend <- c(vs_legend,
                       "Gaussian, sd = sd(BAK)")
        
        vc_cols   <- c(vc_cols, 
                       "black")
      }
      
      legend(#xpd = TRUE, # to enable printing outside plot
        x      = str_location, 
        legend = vs_legend, 
        col    = vc_cols,
        lty    = 1,
        # pt.cex = c(1.5),
        # title  = "Indicative:",
        bg = "white"
      )
    }
  } # END if (bo_show_plots) 
  

  
  
  
  if (bo_show_plots)
  {
  
  
    #### Residuals ####
    
    plot(
      li_val_stats_BAK$vn_error,
      type = 'l',
      col = col_BAK,
      main = "Prediction residuals",
      ylab = "",
      xlab = "Distance"
      )
    
    
    if(bo_show_REML)
    {
      lines(
        li_val_stats_REML$vn_error,
        type = 'l',
        col = col_reml
        
      )
    }
    
    
    if(bo_show_basel)
    {
      lines(
        li_val_stats_baseline$vn_error,
        type = 'l',
        col = col_basel
        
      )
    }
    
    grid()
    
    draw_legend(bo_show_REML, bo_show_basel)
    
    
    #### Residuals density ####
    
    plot(
      density(li_val_stats_BAK$vn_error),
      col = col_BAK,
      main = "Prediction residuals density"
    )
    
    if(bo_show_REML)
    {
      lines(
        density(li_val_stats_REML$vn_error),
        col = col_reml 
      )
    }
    
    if(bo_show_basel)
    {
      lines(
        density(li_val_stats_baseline$vn_error),
        col = col_basel
        
      )
    }    
    
    curve(dnorm(x, 
                mean = 0, #mean(li_val_stats_BAK$vn_error),
                sd   = sd(li_val_stats_BAK$vn_error)
                ),
          # from = n_start_val,
          # to   = n_max_val,
          add  = TRUE,
          col  = "black")
    
    
    grid()
    
    draw_legend(bo_show_REML,bo_show_basel, bo_show_gauss=TRUE)
    
    
    #### Residuals Q-Q plot ####
    
    qqnorm(li_val_stats_BAK$vn_error, 
           col  = col_BAK,
           main = "QQ plot residuals" )
    
    if(bo_show_REML)
    {
      points(
        qqnorm(li_val_stats_REML$vn_error, plot.it = FALSE),
        col = col_reml,
        pch = 4
      )
    }
    
    if(bo_show_basel)
    {
      points(
        qqnorm(li_val_stats_baseline$vn_error, plot.it = FALSE),
        col = col_basel,
        pch = 3
      )
    }      
    
    
    qqline(li_val_stats_BAK$vn_error)
    
    grid()
    
    draw_legend(bo_show_REML,bo_show_basel)
    
    #### Standardized squared error ####
    # Standardised squared error 
    # See Lark, 2000, https://doi.org/10.1046/j.1365-2389.2000.00345.x
    
    n_start_val <- 0.01
    n_max_val   <- max(li_val_stats_BAK$vn_stse)
    
    plot(
      density(li_val_stats_BAK$vn_stse, from = n_start_val, to = n_max_val),
      col = col_BAK,
      main = "Standardized squared pred. error density"
    )
    
    if(bo_show_REML)
    {
      lines(
        density(li_val_stats_REML$vn_stse, 
                from = n_start_val, 
                to = n_max_val),
        col = col_reml
      )
    }
    
    curve(dchisq(x, df=1),
          from = n_start_val,
          to   = n_max_val,
          add  = TRUE,
          col  = "black")
    
    draw_legend(bo_show_REML, 
                bo_show_basel = FALSE,
                str_location  = "topright", 
                bo_show_chisq = TRUE)
  } # END if (bo_show_plots)
  
  
  #### export table with summary ####
  ## Table 2 in paper
  
  df_validation_results <- data.frame(  BAK =  unlist(li_val_stats_BAK[3:9]) 
                                        # start with 3 to avoid a lot of useless data going around
  )
  if (bo_show_REML)
  {
    df_validation_results <- cbind (
      df_validation_results,
      REML = unlist(li_val_stats_REML[3:9])
    )
    
  }
  
  if (bo_show_basel)
  {
    df_validation_results <- cbind (
      df_validation_results,
      Basel = unlist(li_val_stats_baseline[3:9])
    )
    
  }
  
  
  
  if (bo_tables_to_latex)
  {  
    
  df_print_table <-   as.data.frame(
                         t(df_validation_results[c(5,2,1,7),c(2,1,3)])
                        )
  str_caption <- "Results single simulation run  with validating on  original data points. 
  \\newline 
  REML: restricted maximum likelihood; 
  StSE: standardised squared error;   
  ME: mean error; 
  RMSE: root mean squared error; max(MPP): maximal found deviation from the mass preserving property.
  The baseline approach does not provide a quantification of prediction uncertainty." 
  
  names(df_print_table) <- 
    c(
      "mean(StSE)",
      "RMSE",
      "ME",
      "max(MMP)"      
    )
  
  rownames(df_print_table) <- 
    c(
      "REML",
      "Bayesian-Uniform",
      "Baseline"      
    )
  
  
  ## get rif of ugly "-0.000" 
  df_print_table$ME <-  ifelse(abs(df_print_table$ME) < 1e-10 , 0,  df_print_table$ME)

  print.xtable(
    xtable(
      df_print_table, 
      digits=c(rep(NA, times=1), rep(3,times=4)   ), 
      caption = str_caption, 
      label="tb:results-single-simulation",
      table.placement =""
    ), 
    type             = "latex",
    include.rownames = TRUE,
    include.colnames = TRUE,
    NA.string        = "-",
    caption.placement="top",
    file             = paste0(str_path_export_table, str_table_file_name) 
  )
  
  } else
  {
    (df_print_table)
  }
  
  return(df_validation_results)
} 
